/*---------------------------------------------------------------------------+
|	This file is part of MuMail, Copyright (c) 1992-1993 by
|	Muhammad M. Saggaf. All rights reserved.
|
|	See the file COPYING (1-COPYING) or the manual page mumail(1)
|	for a full statement of rights and permissions.
+---------------------------------------------------------------------------*/

#include <X11/Intrinsic.h>
#include <X11/Xaw/AsciiText.h>
#include <stdio.h>
#include <time.h>

extern char
                *GetTextSourceOfBodyBox(),

                *ConstructLetter(),

                *FormatParagraph(),
                *GetTextSource(),
                *GetCurrentTextSelection(),
                *TextFindHeaderField(),

                *BuildAddress(),

                *FileInMuMailDir(),
                *XMailerMessage();


Boolean
               BodyBoxEditable();

extern int

                ExternalEditFileOfBodyBox(),

                PrepareDraft(),

                OpenFolder(),
                CbSaveFolder(),
                CommitChanges(),
                Exit(),

                DeleteLetterProc(),
                UndeleteLetterProc(),
                ComposeLetter(),
                ReplyToLetter(),
                ForwardLetter(),
                ExternalComposeLetter(),
                CopyLetter(),
                RefileLetterProc(),
                DeleteLetter(),
                UndeleteLetter(),
                LetterInfo(),

                DialogInputAndDispatch(),
                BooleanInputAndDispatch(),

                SetCurLetterNumber(),

                ReadParseMailFile(),

                GetLettersFromFolder(),
                ActualOpenFolder(),
                SaveFolder(),

                TocSetLetterFlag(),                

                DisplayCurLetterPartial(),
                DisplayCurLetterWhole(),

                DisplayLetterInBodyBox(),

                InsertText(),        
                InsertFile();

extern XawTextPosition 
                FindEndOfLine(),
                FindEndOfCurrentLine(),
                FindEndOfBuffer(),

                SearchForText(),
                ReplaceTextByPos(),
                ReplaceText();

  
extern void

                CloseWindow(),

                EditFileInBodyBox(),

                EditDraft(),
                CancelDraftCallback(),
                QuoteRnStyleCallback(),
                QuoteScStyleProc(),
                InsertLetterWhole(),
                InsertSignatureCallback(),

                OpenFolderCallback(),
                OpenFolderCallback1_1(),
                QuickExit(),

                ViewLetterCallback(),
                DisplayCurLetterPartialCallback(),
                TagUntagLetterCallback(),
                RetagLettersCallback(),
                DeleteLetterCallback(),
                UndeleteLetterCallback(),
                CopyLetterCallback(),
                RefileLetterCallback(),
                WriteLetterCallback(),

                DelayedCallback(),

                ScrollTextToLine(),

                ScrollTocToLine(),
                TocHighlightLetter(),


                CallAction(),
                CallSimpleAction(),
                MoveToStartOfLine(),
                MoveToEndOfLine(),
                MoveToNextLine(),
                MoveToPrevLine(),
                MoveToStartOfBuffer(),
                MoveToEndOfBuffer(),
                MoveToStartOfParagraph(),

                ClearCutBufferAction(),
                SetCurLetterNumberAction(),
                ViewCurLetterAction(),
                IncCurLetterNumberAction(),
                DecrCurLetterNumberAction(),

                CreateMainMenu(),

                DisplayLogo(),
                Message(),

                DisplayBufferInBodyBox(),
                DisplayFileInBodyBox(),
                HighlightCurrentLine(),
                HighlightToEndOFLine(),

                MarkLetterAsRepliedTo(),

                MakeLetterIndex(),
                RedisplayToc();

#define MAX_SCREENS 10
#define MAX_LETTERS 256
#define MAX_HEADERS 256

#define INFOLDER_NAME      "MUINBOX"
#define DRAFT_NAME         "draft"
#define HEADERS_FILE       "headers"
#define SIG_FILE           ".signature"

#define ERROR          "An Error Occured\n"

#define FROM_TITLE         "From:"
#define TO_TITLE           "To:"
#define SUBJECT_TITLE      "Subject:"
#define DATE_TITLE         "Date:"
#define READ_TITLE         "Read:"
#define REPLIED_TITLE      "Replied:"
#define FCC_TITLE          "Fcc:"

#define curLetterN curLetN
#define DRAFT_NO_REPLY     -1

#define TOC_FLAG_POS       3
#define TOC_CLEAR_FLAG     " "
#define TOC_UNREAD_FLAG    "n"
#define TOC_READ_FLAG      TOC_CLEAR_FLAG
#define TOC_DEL_FLAG       "d"
#define TOC_REPL_FLAG      "r"
#define TOC_TAG_FLAG       "t"

#define LETTER_MAKE_WHOLE            1
#define LETTER_MAKE_PARTIAL          2
#define LETTER_MAKE_BODY             3
#define LETTER_MAKE_WITH_DELIMITER   4

#define LETTER_COMP_DELIMITER "-- letter body follows --"

#define MSG_CLEAR_NEVER     0
#define MSG_CLEAR_DEFAULT  15	/* in seconds */

#define QUOTE_SC_STYLE_IND    1
#define QUOTE_SC_STYLE_UNIND  2

#define BUF10       10
#define BUF80       80
#define BUF256      256

#define CNULL       ('\0')
#define SPACE       (' ')
#define NEWLINE     ('\n')

/*---------------------------------------------------------------------------+
| Macros.
+---------------------------------------------------------------------------*/

#define OpenStream(fp,name,mode,retValue)\
  {if ((fp = fopen(name, mode)) == NULL)\
     {MuPError(FmtString1("Could not open file ``%s''", name));\
	  return retValue;}}

#define SetupTextBlock(var,text){\
  textBlock.ptr = text; textBlock.firstPos = 0;\
  textBlock.length = strlen(textBlock.ptr);\
  textBlock.format = FMT8BIT;}

#define EditMode(scn,mode)\
  {scn->edit.active = mode;\
   XtVaSetValues(scn->edit.buttonW, XtNsensitive, mode, NULL);}
#define DraftMode(scn,mode)\
  {scn->draft.active = mode;\
   XtVaSetValues(scn->draft.buttonW, XtNsensitive, mode, NULL);\
   EditMode(scn,mode);\
   XtVaSetValues(scn->edit.cancelW, XtNsensitive, !mode, NULL);}

#define CheckIfLetterOk(scn,letterN,ret) \
  {if (letterN < 0 || letterN > scn->folder.numLetters - 1) return ret;}

#define ErrorIfEditOrDialogMode(scn,ret)\
  {if (scn->edit.active || scn->dialog.active) return ret;}
#define ErrorIfBodyBoxNotEditable(scn,ret)\
  {if (!BodyBoxEditable(scn)) return ret;}

/*---------------------------------------------------------------------------+
| Structures.
+---------------------------------------------------------------------------*/

typedef struct {
  String          name;
  String          buffer;
  int             numLetters;
  int             curLetN;
  Boolean         changed;
  int             numTagged;

  Boolean         unused1;
  Boolean         unused2;
  Boolean         unused3;
  Boolean         unused4;
} Folder;

typedef struct {
  String          fullS;
  time_t          timeU;
} Date;

typedef struct {
  int             number;
  long            totalLength;

  String          delimiter;

  String          from;
  String          to;
  String          subject;
  Date            date;

  String          otherHeaders;
  String          content;
  
  String          readS;
  String          repliedS;

  Boolean         read;
  Boolean         deleted;
  Boolean         replied;
  Boolean         tagged;
  Boolean         retag;

  Boolean         unused1;
  Boolean         unused2;
  Boolean         unused3;
  Boolean         unused4;

} Letter;

typedef struct {
  Widget          buttonW;
  String          name;
  Boolean         active;
  Boolean         replLetN;

  Boolean         unused1;
  Boolean         unused2;
  Boolean         unused3;
  Boolean         unused4;
} Draft;

typedef struct {
  Widget          valueW;
  Widget          yesW;
  Widget          noW;
  Widget          cancelW;
  Boolean         active;

  Boolean         unused1;
  Boolean         unused2;
} Dialog;

typedef struct {
  Widget          buttonW;
  Widget          saveW;
  Widget          cancelW;

  String          fileName;
  Boolean         active;

  Boolean         unused1;
  Boolean         unused2;
  Boolean         unused3;
  Boolean         unused4;
} Edit;

typedef struct {
  XtPointer field[6];
} ClientDataRec;

typedef struct {
  int             number;

  Widget          topLevelW;
  Widget          parentW;
  Widget          tocW;
  Widget          labelW;
  Widget          bodyW;

  Folder          folder;
  Letter          *letterList[MAX_LETTERS];
  Draft           draft;

  String          tocS;  
  Dialog          dialog;
  Edit            edit;
  ClientDataRec   clientDataRec;

  Boolean         unused1;
  Boolean         unused2;
  Boolean         unused3;
  Boolean         unused4;

} SCREEN;

typedef Draft DRAFT;
typedef Edit EDIT;
typedef Dialog DIALOG;
typedef Letter LETTER;
typedef Folder FOLDER;
typedef ClientDataRec _clientDataRec;

extern SCREEN          *scnList[];
extern int             numScreens;

#define VERSION    "1"
#define REVISION   "2b"

extern SCREEN
                *AddScreen(),
                *NextScreen(),
                *WidgetToScn(),
                *NewWindowProc();
