/*---------------------------------------------------------------------------+
|	This file is part of MuMail, Copyright (c) 1992-1993 by
|	Muhammad M. Saggaf. All rights reserved.
|
|	See the file COPYING (1-COPYING) or the manual page mumail(1)
|	for a full statement of rights and permissions.
+---------------------------------------------------------------------------*/

#include "config.h"
#include <X11/Intrinsic.h>
#include <X11/StringDefs.h>
#include <X11/Xaw/AsciiText.h>
#include <stdio.h>
#include <unistd.h>
#include <ctype.h>
#include "MuWin.h"
#include "MuGeneric.h"
#include "MmDecl.h"

extern String
                FindHeaderFieldFromFile();

extern void
                InsertLetterBodyCallback();

String
FindHeaderFieldFromTextW(textW, headerTitle, endPos)
	 Widget                   textW;
	 String                   headerTitle;
	 XawTextPosition          endPos;
{
  XawTextPosition          findPos;

  if ((findPos = SearchForText(textW, headerTitle, 0, endPos, XawsdRight)) 
	  == XawTextSearchError) return NULL;

  return GetTextSource(textW, findPos + strlen(headerTitle) + 1, 
					   FindEndOfLine(textW, findPos) - 1);
}

int
FileDraftIfFcc(scn, headersEndPos)
	 SCREEN                   *scn;
	 XawTextPosition          headersEndPos;
{
  String                   folderName,
                           draftBuf, date;
  FILE                     *folderFP;
  XawTextPosition          findPos, lineEndPos;

  if ((findPos = SearchForText(scn->bodyW, FCC_TITLE, 0, headersEndPos, 
							   XawsdRight)) == XawTextSearchError) 
	return 0;

  folderName = GetTextSource(scn->bodyW, findPos + strlen(FCC_TITLE) + 1, 
					   (lineEndPos = FindEndOfLine(scn->bodyW, findPos)) - 1);
  ReplaceTextByPos(scn->bodyW, findPos, lineEndPos + 1, "");

  OpenStream(folderFP, FileInMailDir(folderName), "a", -1);

  date = CurrentLocalTimeString();
  draftBuf = GetTextSource(scn->bodyW, 0, FindEndOfBuffer(scn->bodyW));
  fprintf(folderFP, "From mailer-mumail %s\n%s %s\n%s\n\n", 
		  date , DATE_TITLE, date, draftBuf);

  fclose(folderFP);
  return 0;
}

int
PrepareDraft(scn, forReply)
	 SCREEN          *scn;
     Boolean         forReply;
{
  Letter          *letter = scn->letterList[scn->folder.curLetN];
  FILE            *draft;
  String          headersFile, headersBuf, headersBufPtr;

  OpenStream(draft, FileInMailDir(scn->draft.name), "w", -1);
  
  if (forReply) 
	fprintf(draft, "%s %s\n%s %s%s\n%s %s\n",
			TO_TITLE, letter->from, 
			SUBJECT_TITLE, strncmp(letter->subject, "Re:", 3) ? "Re: " : "", 
			letter->subject,
			"In-reply-to: Your message of",  letter->date.fullS);
  else fprintf(draft, "%s \n%s \n", 
			   TO_TITLE, 
			   SUBJECT_TITLE);

  /* Just to make sure the custom headers have no 
	 leading or trailing blank lines */

  if (access(headersFile = FileInMuMailDir(HEADERS_FILE), R_OK) == 0 &&
	  (headersBuf = CopyFileToBuffer(headersFile))) {

	headersBufPtr = strend(headersBuf) - 1;
	while(isspace(*headersBufPtr)) headersBufPtr--;
	*++headersBufPtr = CNULL;

	headersBufPtr = headersBuf;
	while(isspace(*headersBufPtr)) headersBufPtr++;

	fprintf(draft, "%s\n", headersBufPtr);
  }

  fprintf(draft, "%s\n", LETTER_COMP_DELIMITER);
  fclose(draft);

  scn->draft.replLetN = forReply ? scn->folder.curLetN : DRAFT_NO_REPLY;
  return 0;
}

void
EditDraft(scn)
	 SCREEN          *scn;
{
  EditFileInBodyBox(scn, FileInMailDir(scn->draft.name));
  DraftMode(scn, True);
}

void 
SendDraftCallback(widget, clientData, callData)
	 Widget             widget;
	 XtPointer          clientData;
	 XtPointer          callData;
{
  SCREEN                   *scn = (SCREEN*)clientData;
  String                   draftName, sendCmd, receipient;
  XawTextPosition          headersEndPos;
  int                      ret;

  if (scn->draft.active == False) return;

  XawTextDisableRedisplay(scn->bodyW);
  draftName = FileInMailDir(scn->draft.name);

  if ((headersEndPos = ReplaceText(scn->bodyW, LETTER_COMP_DELIMITER, 
								   XMailerMessage(), 0)) == XawEditError)
	Message(scn, 
			"Malformed draft: no header delimiter, not sending, draft left", 
			MSG_CLEAR_DEFAULT);
  
  else if ((receipient = BuildAddress(FindHeaderFieldFromTextW(scn->bodyW, 
		   TO_TITLE, headersEndPos))) == NULL)
	Message(scn, "Unable to determine receipient, not sending, draft left", 
			MSG_CLEAR_DEFAULT);

  else if ((ret = FileDraftIfFcc(scn, headersEndPos)) && 0);
  
  else if (XawAsciiSaveAsFile(XawTextGetSource(scn->bodyW), draftName) 
		   == False)
	Message(scn, 
			FmtString1("Unable to save draft %s, not sending, draft left", 
					   draftName),	MSG_CLEAR_DEFAULT);

  else if (MuSystem(sendCmd = FmtString3("cat %s | %s \"%s\"", draftName, 
										 SENDMAIL_PROG, receipient)) < 0) {
	Message(scn, 
       "Draft send command failed, letter may not have been sent, draft left",
			MSG_CLEAR_DEFAULT);
	MuError(FmtString1("Command ``%s'' failed", sendCmd));
  } 

  else {
	BackupFile(draftName);
	unlink(draftName);

	if (scn->draft.replLetN != DRAFT_NO_REPLY) 
	  MarkLetterAsRepliedTo(scn, scn->draft.replLetN);

	Message(scn, FmtString2("%s%s", "Letter sent", (ret < 0) ? " (errors)" :
							""), MSG_CLEAR_DEFAULT);
	printf("(debug) send command = ``%s''\n", sendCmd);
  }

  DraftMode(scn, False);
  DisplayCurLetterPartial(scn);
}

void
CancelDraftCallback(widget, clientData, callData)
	 Widget             widget;
	 XtPointer          clientData;
	 XtPointer          callData;
{
  SCREEN          *scn = (SCREEN*)clientData;

  if (scn->draft.active == False);

  DraftMode(scn, False);
  DisplayCurLetterPartial(scn);

  Message(scn, FmtString1("Composition aborted, draft kept as %s", 
						  scn->draft.name), MSG_CLEAR_DEFAULT);
}

/*---------------------------------------------------------------------------+
| rn-style quote routine.
+---------------------------------------------------------------------------*/

void 
QuoteRnStyleCallback(widget, clientData, callData)
	 Widget             widget;
	 XtPointer          clientData;
	 XtPointer          callData;
{
  SCREEN          *scn = (SCREEN*)clientData;
  LETTER          *letter = scn->letterList[scn->folder.curLetterN];
  String          lineStartPtr, lineEndPtr;
  char            c;

  XawTextDisableRedisplay(scn->bodyW);
  InsertText(scn->bodyW, FmtString1("\n>>>>> %s wrote:\n>\n", letter->from)); 

  for (lineStartPtr = letter->content; 
	   (lineEndPtr = strchr(lineStartPtr, '\n')); 
	   lineStartPtr = lineEndPtr) {

	c = *(++lineEndPtr);
	*lineEndPtr = CNULL;
	InsertText(scn->bodyW, ">");
	InsertText(scn->bodyW, lineStartPtr);
	*lineEndPtr = c;
  }
  InsertText(scn->bodyW, "\n");
  XawTextEnableRedisplay(scn->bodyW);
}

/*---------------------------------------------------------------------------+
| SuperCite-style quote routines.
+---------------------------------------------------------------------------*/

Boolean
OkToQuoteLine(line)
	 String          line;
{
  String          firstSpacePtr;

  while(*line && isspace(*line)) line++;

  if (*line == CNULL || *line == '>' || 
	  ((firstSpacePtr = strchr(line, SPACE)) && 
	   *(firstSpacePtr - 1) == '>'))
	return False;

  return True;
}

void 
QuoteScStyleUnfmtProc1(scn, buffer, initials, indent)
	 SCREEN          *scn;
	 String          buffer,
	                 initials;
	 String          indent;
{
  LETTER  *letter = scn->letterList[scn->folder.curLetterN];
  String  lineStartPtr, lineEndPtr;
  char    c;

  XawTextDisableRedisplay(scn->bodyW);
  InsertText(scn->bodyW, FmtString2("\n>>>>> %s == %s wrote:\n\n", initials, 
									letter->from)); 

  for (lineStartPtr = buffer; 
	   (lineEndPtr = strchr(lineStartPtr, '\n')); 
	   lineStartPtr = lineEndPtr) {

	c = *(++lineEndPtr);
	*lineEndPtr = CNULL;

	if(OkToQuoteLine(lineStartPtr))
	  InsertText(scn->bodyW, FmtString2("%s%s> ", indent, initials));
	
	InsertText(scn->bodyW, lineStartPtr);
	*lineEndPtr = c;
  }
  InsertText(scn->bodyW, "\n");
  XawTextEnableRedisplay(scn->bodyW);
}

void 
QuoteScStyleUnfmtCallback1(widget, clientData, callData)
	 Widget    widget;
	 XtPointer clientData;
	 XtPointer callData;
{
  SCREEN          *scn = (SCREEN*)clientData;
  LETTER          *letter = scn->letterList[scn->folder.curLetterN];
  QuoteScStyleUnfmtProc1(scn, letter->content, (String)callData, "");
}

#define MAXWIDTH 60
#define INDENT   "\t"
#define MAX_PARAGRAPHS 256

void 
QuoteScStyleFmtCallback1(widget, clientData, callData)
	 Widget             widget;
	 XtPointer          clientData;
	 XtPointer          callData;
{
  SCREEN          *scn = (SCREEN*)clientData;
  Letter          *letter = scn->letterList[scn->folder.curLetterN];
  Boolean         noQuotePar;
  char            c;
  String          parStartPtr, lineStartPtr, lineEndPtr,
                  par[MAX_PARAGRAPHS], buffer;
  int             parN, i;

  for (parN = 0, lineEndPtr = parStartPtr = letter->content; 
	   lineEndPtr && parN < MAX_PARAGRAPHS; parStartPtr = lineEndPtr) {

	for (lineStartPtr = parStartPtr; 
		 *lineStartPtr == '\n'; lineStartPtr++);
	
	for (noQuotePar = False; 
		 (lineEndPtr = strchr(lineStartPtr, '\n')) && 
		 *(lineEndPtr+1) != '\n';
		 lineStartPtr = lineEndPtr) {

	  if (noQuotePar) {lineStartPtr = ++lineEndPtr; continue;}

	  c = *(++lineEndPtr);
	  *lineEndPtr = CNULL;

	  if(!OkToQuoteLine(lineStartPtr))
		noQuotePar = True;
	  
	  *lineEndPtr = c;
	}

	if (lineEndPtr) {
	  c = *++lineEndPtr;
	  *lineEndPtr = CNULL;
	  par[parN++] =  noQuotePar ? XtNewString(parStartPtr) :
		FormatParagraph(parStartPtr, MAXWIDTH - strlen((String)callData));
	  *lineEndPtr = c;
	}
	else
	  par[parN++] =  noQuotePar ? XtNewString(parStartPtr) :
		FormatParagraph(parStartPtr, MAXWIDTH - strlen((String)callData));
  }
  par[parN] = NULL;

  buffer = (String)XtMalloc(strlen(letter->content)+1);

  for (i = 0, *buffer = CNULL; i < parN; i++) {
	strcat(buffer, par[i]);
	XtFree(par[i]);
  }
  
  QuoteScStyleUnfmtProc1(scn, buffer, (String)callData, INDENT);
}

void 
QuoteScStyleProc(scn, style)
	 SCREEN          *scn;
	 int             style;
{
  ErrorIfBodyBoxNotEditable(scn,);

  Message(scn, "Enter initials above", MSG_CLEAR_DEFAULT);
  if (style == QUOTE_SC_STYLE_UNIND)
	DialogInputAndDispatch(scn, QuoteScStyleUnfmtCallback1, (XtPointer)scn);
  else if (style == QUOTE_SC_STYLE_IND)
	DialogInputAndDispatch(scn, QuoteScStyleFmtCallback1, (XtPointer)scn);
}

void 
QuoteScStyleUnfmtCallback(widget, clientData, callData)
	 Widget    widget;
	 XtPointer clientData;
	 XtPointer callData;
{
  QuoteScStyleProc((SCREEN*)clientData, QUOTE_SC_STYLE_UNIND);
}

void 
QuoteScStyleFmtCallback(widget, clientData, callData)
	 Widget    widget;
	 XtPointer clientData;
	 XtPointer callData;
{
  QuoteScStyleProc((SCREEN*)clientData, QUOTE_SC_STYLE_IND);
}

void 
QuoteIIICallback(widget, clientData, callData)
	 Widget    widget;
	 XtPointer clientData;
	 XtPointer callData;
{
  SCREEN *scn = (SCREEN*)clientData;
  LETTER *letter = scn->letterList[scn->folder.curLetterN];

  XawTextDisableRedisplay(scn->bodyW);
  InsertText(scn->bodyW, FmtString("\n>>> %s wrote:\n", letter->from, 
									  "", ""));
  /* Redisplay will be enabled there */
  InsertLetterBodyCallback(widget, clientData, callData);
}

void
InsertLetterBodyCallback(widget, clientData, callData)
	 Widget    widget;
	 XtPointer clientData;
	 XtPointer callData;
{
  SCREEN *scn = (SCREEN*)clientData;
  LETTER *letter = scn->letterList[scn->folder.curLetterN];
  String  letterBuffer;

  XawTextDisableRedisplay(scn->bodyW);

  letterBuffer = ConstructLetter(letter, LETTER_MAKE_BODY);
  InsertText(scn->bodyW, letterBuffer);
  XtFree(letterBuffer);

  XawTextEnableRedisplay(scn->bodyW);


/*  InsertText(scn->bodyW, letter->content); 
this would suffice, but I want consistency */
}

void
InsertLetterPartialCallback(widget, clientData, callData)
	 Widget    widget;
	 XtPointer clientData;
	 XtPointer callData;
{
  SCREEN *scn = (SCREEN*)clientData;
  LETTER *letter = scn->letterList[scn->folder.curLetterN];
  String  letterBuffer;

  XawTextDisableRedisplay(scn->bodyW);

  letterBuffer = ConstructLetter(letter, LETTER_MAKE_PARTIAL);
  InsertText(scn->bodyW, letterBuffer);
  XtFree(letterBuffer);

  XawTextEnableRedisplay(scn->bodyW);
}

void
InsertLetterWhole(scn)
  SCREEN *scn;
{
  LETTER *letter = scn->letterList[scn->folder.curLetterN];
  String  letterBuffer;

  XawTextDisableRedisplay(scn->bodyW);
  letterBuffer = ConstructLetter(letter, LETTER_MAKE_WHOLE);
  InsertText(scn->bodyW, letterBuffer);
  XtFree(letterBuffer);
}

void
InsertLetterWholeCallback(widget, clientData, callData)
	 Widget    widget;
	 XtPointer clientData;
	 XtPointer callData;
{
  SCREEN *scn = (SCREEN*)clientData;
  InsertLetterWhole(scn);
  XawTextEnableRedisplay(scn->bodyW);
}

void 
InsertSignatureCallback(widget, clientData, callData)
	 Widget             widget;
	 XtPointer          clientData;
	 XtPointer          callData;
{
  SCREEN *scn = (SCREEN*)clientData;
  InsertFile(scn->bodyW, FileInHomeDir(SIG_FILE));
}
