/*---------------------------------------------------------------------------+
|	This file is part of MuMail, Copyright (c) 1992-1993 by
|	Muhammad M. Saggaf. All rights reserved.
|
|	See the file COPYING (1-COPYING) or the manual page mumail(1)
|	for a full statement of rights and permissions.
+---------------------------------------------------------------------------*/

#include <X11/Intrinsic.h>
#include <X11/StringDefs.h>
#include <X11/Xaw/Text.h>
#include <X11/Xaw/AsciiText.h>
#include <X11/Xaw/TextSrc.h>
#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <sys/wait.h>
#include "MuWin.h"
#include "MuGeneric.h"
#include "MmDecl.h"

int
OpenFolder(scn, folderName)
	 SCREEN *scn;
	 String folderName;
{
  if (GetLettersFromFolder(scn, folderName) < 0) return -1;

  scn->folder.changed = False;
  if (folderName != scn->folder.name) {
	XtFree(scn->folder.name);
	scn->folder.name = XtNewString(folderName);
  }

  MakeLetterIndex(scn);
  RedisplayToc(scn);
  return 0;
}

/*---------------------------------------------------------------------------+
| Incorporate new mail routines.
+---------------------------------------------------------------------------*/

int
IncorporateNewMailProc(scn)
	 SCREEN          *scn;
{
  static String          mailBox;
  FILE                   *mailBoxFP;
  String                 mailBoxBuf;
  int                    len, retStatus;

  if (mailBox == NULL) mailBox = getenv("MAIL");
  if (mailBox == NULL) {
	Message(scn, "Could not get mailbox name, aborting", MSG_CLEAR_DEFAULT);
	return -1;
  }

  if ((mailBoxFP = fopen(mailBox, "r")) == NULL) {
	Message(scn, "No mailbox", MSG_CLEAR_DEFAULT);
	return -1;
  }

  if ((mailBoxBuf = CopyStreamToBuffer(mailBoxFP)) == NULL) {
	Message(scn, "Could not read Mailbox", MSG_CLEAR_DEFAULT);
	XtFree(mailBoxBuf);
	fclose(mailBoxFP);
	return -1;
  }
  
  if ((len = strlen(mailBoxBuf)) == 0) {
	Message(scn, "Mailbox is empty", MSG_CLEAR_DEFAULT);
	XtFree(mailBoxBuf);
	fclose(mailBoxFP);
	return -1;
  }

  retStatus = AppendBufferToFile(FileInMailDir(INFOLDER_NAME), mailBoxBuf);

  fclose(mailBoxFP);
  XtFree(mailBoxBuf);

  if (retStatus < 0) return retStatus;

  if (truncate(mailBox, 0) < 0) {
	Message(scn, "Could not empty mailbox", MSG_CLEAR_DEFAULT);
	MuPError(FmtString1("Could not reuncate file %s", mailBox));
  }

  return OpenFolder(scn, INFOLDER_NAME);
}

void
IncorporateNewMailCallback1(widget, clientData, callData)
	 Widget             widget;
	 XtPointer          clientData;
	 XtPointer          callData;
{
  SCREEN          *scn = (SCREEN*)clientData;
  
  if (IncorporateNewMailProc(scn) >= 0) 
	Message(scn, 
      FmtString1("New mail incorporated to folder %s, now the current folder", 
				 scn->folder.name), MSG_CLEAR_DEFAULT);
}

void
IncorporateNewMailCallback(widget, clientData, callData)
	 Widget             widget;
	 XtPointer          clientData;
	 XtPointer          callData;
{
  SCREEN          *scn = (SCREEN*)clientData;

  if (scn->folder.changed == False)
	{IncorporateNewMailCallback1(widget, clientData, callData);
	 return;}

  if (BooleanInputAndDispatch(scn, IncorporateNewMailCallback1, clientData) 
	  < 0) return;
  Message(scn, "Changes were made to current folder. Discard them?", 
		  MSG_CLEAR_DEFAULT);
}

/*---------------------------------------------------------------------------+
| Open folder routines.
+---------------------------------------------------------------------------*/

void
OpenFolderProc1_1(scn, folderName)
	 SCREEN          *scn;
	 String          folderName;
{
  if (OpenFolder(scn, folderName) < 0)
	Message(scn, FmtString1("Error in opening folder %s, folder not opened", 
							folderName), MSG_CLEAR_DEFAULT);
  else
	Message(scn, FmtString1("Folder %s loaded", scn->folder.name), 
			MSG_CLEAR_DEFAULT);
}

void
OpenFolderCallback1_1(widget, clientData, callData)
	 Widget    widget;
	 XtPointer clientData;
	 XtPointer callData;
{
  _clientDataRec  *clientDataRec = (_clientDataRec*)clientData;

  OpenFolderProc1_1((SCREEN*)clientDataRec->field[0], 
					(String)clientDataRec->field[1]);
}

void 
OpenFolderCallback1(widget, clientData, callData)
	 Widget    widget;
	 XtPointer clientData;
	 XtPointer callData;
{
  SCREEN                *scn = (SCREEN*)clientData;
  static _clientDataRec clientDataRec;

  clientDataRec.field[0] = (XtPointer)scn;
  clientDataRec.field[1] = (XtPointer)callData;
  
  if (scn->folder.changed)
	if (BooleanInputAndDispatch(scn, OpenFolderCallback1_1, 
								(XtPointer)&clientDataRec) < 0)
	  return;
	else
	  Message(scn, "Changes were made to current folder. Discard them?", 
			  MSG_CLEAR_DEFAULT);
  else
	OpenFolderCallback1_1(widget, (XtPointer)&clientDataRec, callData);
}

void 
OpenFolderCallback(widget, clientData, callData)
	 Widget    widget;
	 XtPointer clientData;
	 XtPointer callData;
{
  SCREEN          *scn = (SCREEN*)clientData;

  if (DialogInputAndDispatch(scn, OpenFolderCallback1, clientData) < 0)
	return;
  Message(scn, "Enter folder name above", MSG_CLEAR_DEFAULT);
}

/*---------------------------------------------------------------------------+
| Re-open folder routine.
+---------------------------------------------------------------------------*/

void 
ReOpenFolderCallback(widget, clientData, callData)
	 Widget    widget;
	 XtPointer clientData;
	 XtPointer callData;
{
  SCREEN *scn = (SCREEN*)clientData;
  OpenFolderCallback1(widget, clientData, (XtPointer)scn->folder.name);
}

/*---------------------------------------------------------------------------+
| Open folder in new screen routines.
+---------------------------------------------------------------------------*/

void 
OpenFolderInNextScreenCallback1(widget, clientData, callData)
	 Widget    widget;
	 XtPointer clientData;
	 XtPointer callData;
{
  SCREEN          *scn = (SCREEN*)clientData, 
                  *nextScn;

  if ((nextScn = NextScreen(scn)) == NULL)
	nextScn = NewWindowProc(scn);

  OpenFolderProc1_1(nextScn, (String)callData);
}

void 
OpenFolderInNextScreenCallback(widget, clientData, callData)
	 Widget    widget;
	 XtPointer clientData;
	 XtPointer callData;
{
  SCREEN          *scn = (SCREEN*)clientData;

  if (DialogInputAndDispatch(scn, OpenFolderInNextScreenCallback1, clientData) 
	  < 0) return;
  Message(scn, "Enter folder name above", MSG_CLEAR_DEFAULT);
}

#if 0
void
UnlinkAndOpenFolderCallback(widget, clientData, callData)
	 Widget    widget;
	 XtPointer clientData;
	 XtPointer callData;
{
  _clientDataRec *clientDataRec = (_clientDataRec*)clientData;
  SCREEN *scn = (SCREEN*)clientDataRec->field[0];

  XtFree(scn->folder.name);
  scn->folder.name = XtNewString((String)clientDataRec->field[1]);

  if (unlink((String)clientDataRec->field[1]) < 0)
	Message(scn, 
	 FmtString("Error in opening folder %s, folder may not have been opened", 
			   scn->folder.name, "", ""), MSG_CLEAR_DEFAULT);
  else
	Message(scn, FmtString("Folder %s loaded", scn->folder.name, "", ""), 
			MSG_CLEAR_DEFAULT);
}

void
DoNewFolderCallback(widget, clientData, callData)
	 Widget    widget;
	 XtPointer clientData;
	 XtPointer callData;
{
  SCREEN *scn = (SCREEN*)clientData;
  static _clientDataRec clientDataRec;

  clientDataRec.field[0] = (XtPointer)scn;
  clientDataRec.field[1] = callData;

  if (access((String)callData, F_OK) < 0)
	{ExecOpenFolderCallback(widget, (XtPointer)&clientDataRec, callData);
	 return;}

  Message(scn, FmtString("Folder %s already exists. Overwrite it?", 
						 (String)callData, "", ""), MSG_CLEAR_DEFAULT);
  BooleanInputAndDispatch(scn, ExecNewFolderCallback, 
						  (XtPointer)&clientDataRec);
}
#endif

void 
NewFolderCallback(widget, clientData, callData)
	 Widget    widget;
	 XtPointer clientData;
	 XtPointer callData;
{
/*  SCREEN *scn = (SCREEN*)clientData;
  return;
  Message(scn, "Enter new folder name above", MSG_CLEAR_DEFAULT);
  DialogInputAndDispatch(scn, DoNewFolderCallback, (XtPointer)scn);*/
}

int
CbSaveFolder(scn)
	 SCREEN          *scn;
{
  String  tmpPtr;

  if (BackupFile(tmpPtr = FileInMailDir(scn->folder.name)) < 0)
	Message(scn, "Error in backing up folder, folder not saved", 
			MSG_CLEAR_DEFAULT);

  else if (SaveFolder(scn) < 0) 
	Message(scn, "Error in saving folder, folder not saved", 
			MSG_CLEAR_DEFAULT);

  else {
	CommitChanges(scn);
	Message(scn, "Folder saved, old copy bakced up", MSG_CLEAR_DEFAULT);
	return 0;
  }
  return -1;
}

void 
SaveFolderCallback(widget, clientData, callData)
	 Widget    widget;
	 XtPointer clientData;
	 XtPointer callData;
{
  CbSaveFolder((SCREEN*)clientData);
}

void
DoSaveFolderAsCallback(widget, clientData, callData)
	 Widget    widget;
	 XtPointer clientData;
	 XtPointer callData;
{
  SCREEN *scn = (SCREEN*)clientData;

  XtFree(scn->folder.name);
  scn->folder.name = XtNewString((String)callData);

  if (SaveFolder(scn) < 0)
	Message(scn, "Error in saving folder, folder not saved", 
			MSG_CLEAR_DEFAULT);
  else Message(scn, FmtString("Folder saved as %s, now the current folder", 
							  scn->folder.name, "", ""), MSG_CLEAR_DEFAULT);
}

void 
SaveFolderAsCallback(widget, clientData, callData)
	 Widget             widget;
	 XtPointer          clientData;
	 XtPointer          callData;
{
  SCREEN          *scn = (SCREEN*)clientData;

  if (DialogInputAndDispatch(scn, DoSaveFolderAsCallback, (XtPointer)scn) < 0)
	return;
  Message(scn, "Enter new folder name above", MSG_CLEAR_DEFAULT);
}

void 
EditFolderCallback(widget, clientData, callData)
	 Widget             widget;
	 XtPointer          clientData;
	 XtPointer          callData;
{
  SCREEN          *scn = (SCREEN*)clientData;

  ErrorIfEditOrDialogMode(scn,);
  EditFileInBodyBox(scn, FileInMailDir(scn->folder.name));
  EditMode(scn, True);

  Message(scn, "For experts only, use caution", MSG_CLEAR_DEFAULT);
}

void 
DoDeleteFolderCallback(widget, clientData, callData)
	 Widget    widget;
	 XtPointer clientData;
	 XtPointer callData;
{
  SCREEN *scn = (SCREEN*)clientData;
  int     retStatus;

  retStatus = BackupFile(FileInMailDir(scn->folder.name));
  if (unlink(FileInMailDir(scn->folder.name)) < 0)
	Message(scn, "Error in deleting folder, folder not deleted", 
			MSG_CLEAR_DEFAULT);
  else if (retStatus < 0)
	Message(scn, "Folder deleted (but backup error", MSG_CLEAR_DEFAULT);
  else
	Message(scn, FmtString("Folder deleted, backup saved as %s~", 
						   scn->folder.name, "", ""), MSG_CLEAR_DEFAULT);
}

void 
DeleteFolderCallback(widget, clientData, callData)
	 Widget    widget;
	 XtPointer clientData;
	 XtPointer callData;
{
  SCREEN *scn = (SCREEN*)clientData;
  int    retStatus;

  if ((retStatus = BooleanInputAndDispatch(scn, DoDeleteFolderCallback, 
										   (XtPointer)scn)) < 0)
	return /*retStatus*/;
  Message(scn, "Delete current folder? Are you really sure?", 
		  MSG_CLEAR_DEFAULT);
}

int
CommitChanges(scn)
	 SCREEN *scn;
{
  /* We don't want to do anything if the folder doesn't have any letters
	 to begin with */
  if (scn->folder.numLetters == 0) return -1;

  MakeLetterIndex(scn);
  RedisplayToc(scn);
  return 0;
}

void 
CommitChangesCallback(widget, clientData, callData)
	 Widget    widget;
	 XtPointer clientData;
	 XtPointer callData;
{
  SCREEN *scn = (SCREEN*)clientData;
  if (CommitChanges(scn) < 0)
	Message(scn, "No letters in folder, changes not commited", 
			MSG_CLEAR_DEFAULT);
  else Message(scn, "Changes to folder commited", MSG_CLEAR_DEFAULT);
}

/*---------------------------------------------------------------------------+
| Sort folder routine.
+---------------------------------------------------------------------------*/

void 
SortFolderCallback(widget, clientData, callData)
	 Widget    widget;
	 XtPointer clientData;
	 XtPointer callData;
{
  SCREEN          *scn = (SCREEN*)clientData;
  Letter          **letter = scn->letterList,
                  *tmpLetter = NULL;
  int             i, j;

  for (i = 0; i < scn->folder.numLetters; i++)
	for (j = i+1; j < scn->folder.numLetters; j++)
	  if (letter[i]->date.timeU > letter[j]->date.timeU) {
		tmpLetter = letter[i];
		letter[i] = letter[j];
		letter[j] = tmpLetter;
	  }

  MakeLetterIndex(scn);
  RedisplayToc(scn);
  if (tmpLetter) scn->folder.changed = True;
}

/*---------------------------------------------------------------------------+
| Search folder routines.
+---------------------------------------------------------------------------*/

void 
SearchFolderCallback1(widget, clientData, callData)
	 Widget             widget;
	 XtPointer          clientData;
	 XtPointer          callData;
{
  SCREEN          *scn = (SCREEN*)clientData;
  Letter          *letter;
  String          letterBuffer;
  Boolean         found;
  int             i;

  for (i = scn->folder.curLetN, found = False;
	   (letter = scn->letterList[i]) && !found; i++) {

	TocHighlightLetter(scn, i);
	letterBuffer = ConstructLetter(letter, LETTER_MAKE_WHOLE);
	if (strstr(letterBuffer, (String)callData)) found = True;
	XtFree(letterBuffer);
  }

  if (!found) {
	Message(scn, "Search string not found", MSG_CLEAR_DEFAULT);
	return;
  }

  scn->folder.curLetN = --i;
  DisplayCurLetterPartialCallback(widget, clientData, callData);
}

void
SearchFolderCallback(widget, clientData, callData)
	 Widget             widget;
	 XtPointer          clientData;
	 XtPointer          callData;
{
  SCREEN          *scn = (SCREEN*)clientData;
  if (DialogInputAndDispatch(scn, SearchFolderCallback1, clientData) < 0)
	return;
  Message(scn, "Enter string to search letters for above", MSG_CLEAR_DEFAULT);
}

/*---------------------------------------------------------------------------+
| Folder info routine.
+---------------------------------------------------------------------------*/

void 
FolderInfoCallback(widget, clientData, callData)
	 Widget    widget;
	 XtPointer clientData;
	 XtPointer callData;
{
  SCREEN          *scn = (SCREEN*)clientData;
  String          message = FmtString0(""),
                  curLetterMsg;

  curLetterMsg = scn->folder.curLetterN < 0 ? "" :
	FmtString1(" (current is #%d)", scn->folder.curLetterN + 1);

  sprintf(message, "Current folder is %s, has %d letters%s%s", 
		  scn->folder.name, scn->folder.numLetters,
		  curLetterMsg, 
		  scn->folder.changed ? ", not saved" : "");
  Message(scn, message, MSG_CLEAR_DEFAULT);
}

void 
RedisplayTocCallback(widget, clientData, callData)
	 Widget    widget;
	 XtPointer clientData;
	 XtPointer callData;
{
  SCREEN *scn = (SCREEN*)clientData;
  RedisplayToc(scn);
  Message(scn, "Table of contents redisplayed", MSG_CLEAR_DEFAULT);
}

void
QuickExit(scn)
	 SCREEN *scn;
{
  XtUnmapWidget(scn->parentW);
  XtDestroyApplicationContext(XtWidgetToApplicationContext(scn->parentW));
  exit(0);
}

void 
DoExitCallback(widget, clientData, callData)
	 Widget    widget;
	 XtPointer clientData;
	 XtPointer callData;
{
  QuickExit((SCREEN*)clientData);
}

int
Exit(scn)
	 SCREEN *scn;
{
  int    retStatus;

  if (scn->folder.changed == False)
	{QuickExit(scn); return 0;}

  if ((retStatus = BooleanInputAndDispatch(scn, DoExitCallback, 
										   (XtPointer)scn)) < 0)
	return retStatus;
  Message(scn, "Changes were made to current folder. Exit without saving?", 
		  MSG_CLEAR_DEFAULT);
  return 0;
}

void 
ExitCallback(widget, clientData, callData)
	 Widget    widget;
	 XtPointer clientData;
	 XtPointer callData;
{
  Exit((SCREEN*)clientData);
}
