/*---------------------------------------------------------------------------+
|	This file is part of MuMail, Copyright (c) 1992-1993 by
|	Muhammad M. Saggaf. All rights reserved.
|
|	See the file COPYING (1-COPYING) or the manual page mumail(1)
|	for a full statement of rights and permissions.
+---------------------------------------------------------------------------*/

#include <X11/Intrinsic.h>
#include <X11/StringDefs.h>
#include <X11/Xaw/AsciiText.h>
#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <sys/wait.h>
#include "MuWin.h"
#include "MuGeneric.h"
#include "MmDecl.h"

void 
MarkLetterAsRead(scn, letterN)
	 SCREEN          *scn;
	 int             letterN;
{
  LETTER          *letter;

  CheckIfLetterOk(scn, letterN,);
  letter = scn->letterList[letterN];

  if (letter->read) return;
  letter->read = True;
  scn->folder.changed = True;

  if (letter->deleted || letter->replied) return;
  TocSetLetterFlag(scn, letterN, TOC_READ_FLAG);
}

void 
MarkLetterAsRepliedTo(scn, letterN)
	 SCREEN          *scn;
	 int             letterN;
{
  LETTER          *letter;

  CheckIfLetterOk(scn, letterN,);
  letter = scn->letterList[letterN];

  letter->replied = True;
  scn->folder.changed = True;

  if (letter->deleted) return;
  TocSetLetterFlag(scn, letterN, TOC_REPL_FLAG);
}

void 
MarkLetterAsTaggedUntagged(scn, letterN)
	 SCREEN          *scn;
	 int             letterN;
{
  LETTER          *letter;

  CheckIfLetterOk(scn, letterN,);
  letter = scn->letterList[letterN];

  if (letter->tagged) {
	letter->tagged = False;
	if (scn->folder.numTagged) scn->folder.numTagged--;
	
	if (letter->deleted) 
	  TocSetLetterFlag(scn, letterN, TOC_DEL_FLAG);
	else if (letter->replied) 	  
	  TocSetLetterFlag(scn, letterN, TOC_REPL_FLAG);
	else if (letter->read == False) 
	  TocSetLetterFlag(scn, letterN, TOC_UNREAD_FLAG);
	else TocSetLetterFlag(scn, letterN, TOC_CLEAR_FLAG);
	
  }

  else {
	letter->tagged = True;
	scn->folder.numTagged++;
	TocSetLetterFlag(scn, letterN, TOC_TAG_FLAG);
  }
}

void 
MarkLetterAsDeleted(scn, letterN)
	 SCREEN          *scn;
	 int             letterN;
{
  LETTER          *letter;

  CheckIfLetterOk(scn, letterN,);
  letter = scn->letterList[letterN];

  letter->deleted = True;
  scn->folder.changed = True;
  TocSetLetterFlag(scn, letterN, TOC_DEL_FLAG);
}

void 
MarkLetterAsUndeleted(scn, letterN)
	 SCREEN          *scn;
	 int             letterN;
{
  LETTER          *letter;

  CheckIfLetterOk(scn, letterN,);
  letter = scn->letterList[letterN];

  letter->deleted = False;
  scn->folder.changed = True;

  if (letter->replied)
	TocSetLetterFlag(scn, letterN, TOC_REPL_FLAG);
  else if (letter->read == False) 
	TocSetLetterFlag(scn, letterN, TOC_UNREAD_FLAG);
  else TocSetLetterFlag(scn, letterN, TOC_CLEAR_FLAG);
}

int
OperateOnTeggedLetters(scn, letterProc, clientData)
	 SCREEN          *scn;
	 int             (*letterProc)();
	 XtPointer       clientData;
{
  Letter       *letter;
  int          i, n;

  for (i = 0, n = 0; (letter = scn->letterList[i]); i++)
	if (letter->tagged && 
		(*letterProc)(scn, letter->number, clientData) >= 0) {
	  MarkLetterAsTaggedUntagged(scn, letter->number);
	  letter->retag = True;
	  n++;
	}
	else letter->retag = False;

  return n;
}

void
OperateOnTeggedLettersCallback(widget, clientData, callData)
	 Widget             widget;
	 XtPointer          clientData;
	 XtPointer          callData;
{
  _clientDataRec  *clientDataRec = (_clientDataRec*)clientData;
  int             numSuccess;

  numSuccess = 
	OperateOnTeggedLetters((SCREEN*)clientDataRec->field[0], 
						   (int(*)())clientDataRec->field[1],
						   clientDataRec->field[2]);

  (*(XtCallbackProc)clientDataRec->field[3])(widget, clientData,
											 (XtPointer)numSuccess); 
}

int
DisplayLetterInBodyBox(scn, letterNumber, letterDisplayFormat)
	 SCREEN         *scn;
	 int             letterNumber,
                     letterDisplayFormat;
{
  String  letterDisplayBuffer;

  CheckIfLetterOk(scn, letterNumber,-1);

  letterDisplayBuffer = ConstructLetter(scn->letterList[letterNumber], 
										letterDisplayFormat);
  DisplayBufferInBodyBox(scn, letterDisplayBuffer);
  XtFree(letterDisplayBuffer);
  return 0;
}

int
DisplayCurLetter(scn, letterDisplayFormat)
	 SCREEN           *scn;
	 int              letterDisplayFormat;
{
  Letter          *letter;
  int             retStatus;

  CheckIfLetterOk(scn, scn->folder.curLetN,-1);
  letter = scn->letterList[scn->folder.curLetN];

  if ((retStatus = DisplayLetterInBodyBox(scn, scn->folder.curLetterN, 
										 letterDisplayFormat)) >= 0)
	MarkLetterAsRead(scn, letter->number);
  return retStatus;
}

int
DisplayCurLetterPartial(scn)
	 SCREEN           *scn;
{
  return DisplayCurLetter(scn, LETTER_MAKE_PARTIAL);
}

int
DisplayCurLetterWhole(scn)
	 SCREEN           *scn;
{
  return DisplayCurLetter(scn, LETTER_MAKE_WHOLE);
}

int
DisplayLetterPartialInNextScreen(scn)
	 SCREEN           *scn;
{
  SCREEN           *nextScn;

  if ((nextScn = NextScreen(scn)) == NULL)
	nextScn = NewWindowProc(scn);
  return DisplayCurLetter(nextScn, LETTER_MAKE_PARTIAL);
}

void 
DisplayCurLetterPartialCallback(widget, clientData, callData)
	 Widget    widget;
	 XtPointer clientData;
	 XtPointer callData;
{
  DisplayCurLetterPartial((SCREEN*)clientData);
}

void 
DisplayCurLetterWholeCallback(widget, clientData, callData)
	 Widget    widget;
	 XtPointer clientData;
	 XtPointer callData;
{
  DisplayCurLetterWhole((SCREEN*)clientData);
}

void 
DisplayLetterPartialInNextScreenCallback(widget, clientData, callData)
	 Widget    widget;
	 XtPointer clientData;
	 XtPointer callData;
{
  DisplayLetterPartialInNextScreen((SCREEN*)clientData);
}

void 
TagUntagLetterCallback(widget, clientData, callData)
	 Widget    widget;
	 XtPointer clientData;
	 XtPointer callData;
{
  SCREEN          *scn = (SCREEN*)clientData;

  CheckIfLetterOk(scn, scn->folder.curLetN,);
  MarkLetterAsTaggedUntagged(scn, 
							 scn->letterList[scn->folder.curLetN]->number);
}

void 
TagAllLettersCallback(widget, clientData, callData)
	 Widget    widget;
	 XtPointer clientData;
	 XtPointer callData;
{
  SCREEN          *scn = (SCREEN*)clientData;
  Letter          *letter;
  int             i;

  for (i = 0; (letter = scn->letterList[i]); i++)
	if (letter->tagged == False)
	  MarkLetterAsTaggedUntagged(scn, letter->number);

  Message(scn, "All letters tagged", MSG_CLEAR_DEFAULT);
}

void 
UntagAllLettersCallback(widget, clientData, callData)
	 Widget    widget;
	 XtPointer clientData;
	 XtPointer callData;
{
  SCREEN          *scn = (SCREEN*)clientData;
  Letter          *letter;
  int             i;

  for (i = 0; (letter = scn->letterList[i]); i++)
	if (letter->tagged)
	  MarkLetterAsTaggedUntagged(scn, letter->number);

  Message(scn, "All letters untagged", MSG_CLEAR_DEFAULT);
}

void 
RetagLettersCallback(widget, clientData, callData)
	 Widget    widget;
	 XtPointer clientData;
	 XtPointer callData;
{
  SCREEN          *scn = (SCREEN*)clientData;
  Letter          *letter;
  int             i, n;

  for (i = 0, n = 0; (letter = scn->letterList[i]); i++)
	if (!(letter->tagged) && letter->retag)
	  {MarkLetterAsTaggedUntagged(scn, letter->number); n++;}

  Message(scn, FmtString1("%d letters retagged", n), MSG_CLEAR_DEFAULT);
}

/*---------------------------------------------------------------------------+
+----------------------------------------------------------------------------/
| Routines to delete/undelete letters.
+---------------------------------------------------------------------------*/

/*---------------------------------------------------------------------------+
| Delete letter routines.
+---------------------------------------------------------------------------*/

void 
DeleteLetterTaggedDispResultCallback(widget, clientData, callData)
	 Widget    widget;
	 XtPointer clientData;
	 XtPointer callData;
{
  _clientDataRec  *clientDataRec = (_clientDataRec*)clientData;

  Message((SCREEN*)clientDataRec->field[0], FmtString1("%d letters deleted", 
		                                 (int)callData), MSG_CLEAR_DEFAULT);
}

int
DeleteLetterProc(scn, letterN, clientData)
	 SCREEN          *scn;
	 int             letterN;
	 XtPointer       clientData;
{
  MarkLetterAsDeleted(scn, letterN);
  return 0;
}

void 
DeleteLetterCallback(widget, clientData, callData)
	 Widget    widget;
	 XtPointer clientData;
	 XtPointer callData;
{
  SCREEN          *scn = (SCREEN*)clientData;

  scn->clientDataRec.field[0] = (XtPointer)scn;
  scn->clientDataRec.field[1] = (XtPointer)DeleteLetterProc;
  scn->clientDataRec.field[2] = callData;
  scn->clientDataRec.field[3] = 
	(XtPointer)DeleteLetterTaggedDispResultCallback;

  if (scn->folder.numTagged) {
	if (BooleanInputAndDispatch(scn, OperateOnTeggedLettersCallback, 
								(XtPointer)&scn->clientDataRec) < 0)
	  return;
	Message(scn, FmtString1("Delete %d tagged letters?", 
							scn->folder.numTagged), MSG_CLEAR_DEFAULT);
	return;
  }

  DeleteLetterProc(scn, scn->folder.curLetN, clientData);
  Message(scn, "Letter deleted", MSG_CLEAR_DEFAULT);
}

/*---------------------------------------------------------------------------+
| Undelete letter routines.
+---------------------------------------------------------------------------*/

void 
UndeleteLetterTaggedDispResultCallback(widget, clientData, callData)
	 Widget    widget;
	 XtPointer clientData;
	 XtPointer callData;
{
  _clientDataRec  *clientDataRec = (_clientDataRec*)clientData;

  Message((SCREEN*)clientDataRec->field[0], FmtString1("%d letters undeleted", 
		                                   (int)callData), MSG_CLEAR_DEFAULT);
}

int 
UndeleteLetterProc(scn, letterN, clientData)
	 SCREEN          *scn;
	 int             letterN;
	 XtPointer       clientData;
{
  MarkLetterAsUndeleted(scn, letterN);
  return 0;
}

void 
UndeleteLetterCallback(widget, clientData, callData)
	 Widget    widget;
	 XtPointer clientData;
	 XtPointer callData;
{
  SCREEN          *scn = (SCREEN*)clientData;

  scn->clientDataRec.field[0] = (XtPointer)scn;
  scn->clientDataRec.field[1] = (XtPointer)UndeleteLetterProc;
  scn->clientDataRec.field[2] = callData;
  scn->clientDataRec.field[3] = 
	(XtPointer)UndeleteLetterTaggedDispResultCallback;

  if (scn->folder.numTagged) {
	if (BooleanInputAndDispatch(scn, OperateOnTeggedLettersCallback, 
								(XtPointer)&scn->clientDataRec) < 0)
	  return;
	Message(scn, FmtString1("Undelete %d tagged letters?", 
							scn->folder.numTagged), MSG_CLEAR_DEFAULT);
	return;
  }

  UndeleteLetterProc(scn, scn->folder.curLetterN, clientData);
  Message(scn, "Letter undeleted", MSG_CLEAR_DEFAULT);
}

int 
ComposeLetter(scn)
	 SCREEN *scn;
{
  int    retStatus; 

  ErrorIfEditOrDialogMode(scn, -1);
  if ((retStatus = PrepareDraft(scn, False)) < 0) return retStatus;

  XawTextDisableRedisplay(scn->bodyW);

  EditDraft(scn);
  MoveToStartOfBuffer(scn->bodyW);
  MoveToEndOfLine(scn->bodyW);

  XawTextEnableRedisplay(scn->bodyW);
  return 0;
}

void 
ComposeLetterCallback(widget, clientData, callData)
	 Widget    widget;
	 XtPointer clientData;
	 XtPointer callData;
{
  ComposeLetter((SCREEN*)clientData);
}

int 
ReplyToLetter(scn)
	 SCREEN *scn;
{
  int    retStatus; 

  ErrorIfEditOrDialogMode(scn, -1);
  if ((retStatus = PrepareDraft(scn, True)) < 0) return retStatus;

  XawTextDisableRedisplay(scn->bodyW);

  EditDraft(scn);
  MoveToEndOfBuffer(scn->bodyW);
  MoveToStartOfLine(scn->bodyW);

  XawTextEnableRedisplay(scn->bodyW);
  return 0;
}

void 
ReplyToLetterCallback(widget, clientData, callData)
	 Widget    widget;
	 XtPointer clientData;
	 XtPointer callData;
{
  ReplyToLetter((SCREEN*)clientData);
}

int
ForwardLetter(scn)
  SCREEN *scn;
{
  int    retStatus; 

  ErrorIfEditOrDialogMode(scn, -1);
  if ((retStatus = PrepareDraft(scn, False)) < 0) return retStatus;

  XawTextDisableRedisplay(scn->bodyW);

  EditDraft(scn);
  MoveToEndOfBuffer(scn->bodyW);
  MoveToStartOfLine(scn->bodyW);
  InsertText(scn->bodyW, "\n------- Forwarded Message\n\n");
  InsertLetterWhole(scn);
  InsertText(scn->bodyW, "\n------- End of Forwarded Message\n");
 
  MoveToStartOfBuffer(scn->bodyW);
  MoveToEndOfLine(scn->bodyW);

  XawTextEnableRedisplay(scn->bodyW);
  return 0;
}

void 
ForwardLetterCallback(widget, clientData, callData)
	 Widget    widget;
	 XtPointer clientData;
	 XtPointer callData;
{
  ForwardLetter((SCREEN*)clientData);
}

void
QuoteRnStyleAndReplyToLetterCallback(widget, clientData, callData)
	 Widget    widget;
	 XtPointer clientData;
	 XtPointer callData;
{
  if (ReplyToLetter((SCREEN*)clientData) < 0) return;
  QuoteRnStyleCallback(widget, clientData, callData);
}

void
QuoteScStyleUnfmtAndReplyCallback(widget, clientData, callData)
	 Widget    widget;
	 XtPointer clientData;
	 XtPointer callData;
{
  if (ReplyToLetter((SCREEN*)clientData) < 0) return;
  QuoteScStyleProc((SCREEN*)clientData, QUOTE_SC_STYLE_UNIND);
}

void
QuoteScStyleFmtAndReplyCallback(widget, clientData, callData)
	 Widget    widget;
	 XtPointer clientData;
	 XtPointer callData;
{
  if (ReplyToLetter((SCREEN*)clientData) < 0) return;
  QuoteScStyleProc((SCREEN*)clientData, QUOTE_SC_STYLE_IND);
}

int 
ExternalComposeLetter(scn)
	 SCREEN *scn;
{
  int    retStatus; 

  ErrorIfEditOrDialogMode(scn, -1);
  if ((retStatus = PrepareDraft(scn, False)) < 0) return retStatus;

  XawTextDisableRedisplay(scn->bodyW);

  ExternalEditFile(FileInMailDir(scn->draft.name));
  EditDraft(scn);

  XawTextEnableRedisplay(scn->bodyW);
  return 0;
}

void 
ExternalComposeLetterCallback(widget, clientData, callData)
	 Widget    widget;
	 XtPointer clientData;
	 XtPointer callData;
{
  ExternalComposeLetter((SCREEN*)clientData);
}

#if 0
void 
ExternalReplyToLetterCallback(widget, clientData, callData)
	 Widget    widget;
	 XtPointer clientData;
	 XtPointer callData;
{
  SCREEN *scn = (SCREEN*)clientData;
  
  if (PrepaeDraft(scn, scn->draftName, True) < 0) return;
  ExternalEditFile(scn, scn->draftName);

  scn->draft.active = True;
  EditFileInBodyBox(scn, scn->draftName);
  DraftMenuAccess(scn, True);
}
#endif

/*---------------------------------------------------------------------------+
+----------------------------------------------------------------------------/
| Routines to move letters between folders/files (Copy, Refile, Write).
+---------------------------------------------------------------------------*/

int
AppendLetterToFile(letter, file)
	 LETTER          *letter;
	 String          *file;
{
  String           letterBuffer;
  int              retStatus;
 
  letterBuffer = ConstructLetter(letter, LETTER_MAKE_WITH_DELIMITER);
  retStatus = AppendBufferToFile(file, letterBuffer);
  XtFree(letterBuffer);
  return retStatus;
}

/*---------------------------------------------------------------------------+
| Copy letter routines.
+---------------------------------------------------------------------------*/

void 
CopyLetterTaggedDispResultCallback(widget, clientData, callData)
	 Widget    widget;
	 XtPointer clientData;
	 XtPointer callData;
{
  _clientDataRec  *clientDataRec = (_clientDataRec*)clientData;

  Message((SCREEN*)clientDataRec->field[0], 
		  FmtString2("%d letters copied to folder %s", (int)callData, 
					 (String)clientDataRec->field[2]), MSG_CLEAR_DEFAULT);
}

int
CopyLetterProc(scn, letterN, folder)
	 SCREEN          *scn;
	 int             letterN;
	 XtPointer       folder;
{
  LETTER          *letter;
  int             retStatus;

  CheckIfLetterOk(scn, letterN,-1);
  letter = scn->letterList[letterN];

  if ((retStatus = AppendLetterToFile(letter, 
									  FileInMailDir((String)folder))) < 0) {
	Message(scn, FmtString1("Error in copying letter %d", letterN+1), 
			MSG_CLEAR_DEFAULT);
	return retStatus;
  }

  return 0;
}

void 
CopyLetterCallback1(widget, clientData, callData)
	 Widget    widget;
	 XtPointer clientData;
	 XtPointer callData;
{
  SCREEN          *scn = (SCREEN*)clientData;

  scn->clientDataRec.field[0] = (XtPointer)scn;
  scn->clientDataRec.field[1] = (XtPointer)CopyLetterProc;
  scn->clientDataRec.field[2] = callData;
  scn->clientDataRec.field[3] = (XtPointer)CopyLetterTaggedDispResultCallback;

  if (scn->folder.numTagged) {
	if (BooleanInputAndDispatch(scn, OperateOnTeggedLettersCallback, 
								(XtPointer)&scn->clientDataRec) < 0)
	  return;
	Message(scn, FmtString2("Copy %d tagged letters to folder %s?", 
							scn->folder.numTagged, (String)callData), 
			MSG_CLEAR_DEFAULT);
	return;
  }

  if (CopyLetterProc(scn, scn->folder.curLetterN, callData) < 0)
	Message(scn, "Error in copying letter, letter may not have been copied", 
			MSG_CLEAR_DEFAULT);
  else
	Message(scn, FmtString1("Letter copied to folder %s", (String)callData), 
			MSG_CLEAR_DEFAULT);
}

void 
CopyLetterCallback(widget, clientData, callData)
	 Widget    widget;
	 XtPointer clientData;
	 XtPointer callData;
{
  SCREEN          *scn = (SCREEN*)clientData;
  if (DialogInputAndDispatch(scn, CopyLetterCallback1, clientData) < 0)
	return;
  Message(scn, "Enter folder name to copy letter(s) to above", 
		  MSG_CLEAR_DEFAULT);
}

/*---------------------------------------------------------------------------+
| Refile letter routines.
+---------------------------------------------------------------------------*/

void 
RefileLetterTaggedDispResultCallback(widget, clientData, callData)
	 Widget    widget;
	 XtPointer clientData;
	 XtPointer callData;
{
  _clientDataRec  *clientDataRec = (_clientDataRec*)clientData;

  Message((SCREEN*)clientDataRec->field[0], FmtString2(
      "%d letters refiled to folder %s, marked as deleted in current folder", 
		 (int)callData, (String)clientDataRec->field[2]), MSG_CLEAR_DEFAULT);
}

int
RefileLetterProc(scn, letterN, folder)
	 SCREEN          *scn;
	 int             letterN;
	 XtPointer       folder;
{
  LETTER          *letter;
  int             retStatus;
  
  CheckIfLetterOk(scn, letterN,-1);
  letter = scn->letterList[letterN];

  if ((retStatus = AppendLetterToFile(letter, 
									  FileInMailDir((String)folder))) < 0) {
	Message(scn, FmtString1("Error in refiling letter %d", letterN+1), 
			MSG_CLEAR_DEFAULT);
	return retStatus;
  }

  DeleteLetterProc(scn, letterN, folder);
  return 0;
}

void 
RefileLetterCallback1(widget, clientData, callData)
	 Widget    widget;
	 XtPointer clientData;
	 XtPointer callData;
{
  SCREEN          *scn = (SCREEN*)clientData;

  scn->clientDataRec.field[0] = (XtPointer)scn;
  scn->clientDataRec.field[1] = (XtPointer)RefileLetterProc;
  scn->clientDataRec.field[2] = callData;
  scn->clientDataRec.field[3] = 
	(XtPointer)RefileLetterTaggedDispResultCallback;

  if (scn->folder.numTagged) {
	if (BooleanInputAndDispatch(scn, OperateOnTeggedLettersCallback, 
								(XtPointer)&scn->clientDataRec) < 0)
	  return;
	Message(scn, FmtString2("Refile %d tagged letters to folder %s?", 
							scn->folder.numTagged, (String)callData), 
			MSG_CLEAR_DEFAULT);
	return;
  }

  if (RefileLetterProc(scn, scn->folder.curLetterN, callData) < 0)
	Message(scn, "Error in refiling letter, letter may not have been refiled", 
			MSG_CLEAR_DEFAULT);
  else
	Message(scn, FmtString1(
      "Letter refiled to folder %s, marked as deleted in current folder", 
							(String)callData), MSG_CLEAR_DEFAULT);
}

void 
RefileLetterCallback(widget, clientData, callData)
	 Widget    widget;
	 XtPointer clientData;
	 XtPointer callData;
{
  SCREEN          *scn = (SCREEN*)clientData;
  if (DialogInputAndDispatch(scn, RefileLetterCallback1, clientData) < 0)
	return;
  Message(scn, "Enter folder name to refile letter(s) to above", 
		  MSG_CLEAR_DEFAULT);
}

/*---------------------------------------------------------------------------+
| Write letter to file routines.
+---------------------------------------------------------------------------*/

void 
WriteLetterTaggedDispResultCallback(widget, clientData, callData)
	 Widget    widget;
	 XtPointer clientData;
	 XtPointer callData;
{
  _clientDataRec  *clientDataRec = (_clientDataRec*)clientData;

  Message((SCREEN*)clientDataRec->field[0], 
		  FmtString2("%d letters written to file %s", (int)callData, 
					 (String)clientDataRec->field[2]), MSG_CLEAR_DEFAULT);
}

int
WriteLetterProc(scn, letterN, folder)
	 SCREEN          *scn;
	 int             letterN;
	 XtPointer       folder;
{
  LETTER          *letter;
  int             retStatus;
  
  CheckIfLetterOk(scn, letterN,-1);
  letter = scn->letterList[letterN];

  if ((retStatus = AppendLetterToFile(letter, (String)folder)) < 0) {
	Message(scn, FmtString1("Error in writing letter %d", letterN+1), 
			MSG_CLEAR_DEFAULT);
	return retStatus;
  }
  
  return 0;
}

void 
WriteLetterCallback1(widget, clientData, callData)
	 Widget    widget;
	 XtPointer clientData;
	 XtPointer callData;
{
  SCREEN          *scn = (SCREEN*)clientData;

  scn->clientDataRec.field[0] = (XtPointer)scn;
  scn->clientDataRec.field[1] = (XtPointer)WriteLetterProc;
  scn->clientDataRec.field[2] = callData;
  scn->clientDataRec.field[3] = (XtPointer)WriteLetterTaggedDispResultCallback;

  if (scn->folder.numTagged) {
	if (BooleanInputAndDispatch(scn, OperateOnTeggedLettersCallback, 
								(XtPointer)&scn->clientDataRec) < 0)
	  return;
	Message(scn, FmtString2("write %d tagged letters to file %s?", 
							scn->folder.numTagged, (String)callData), 
			MSG_CLEAR_DEFAULT);
	return;
  }

  if (WriteLetterProc(scn, scn->folder.curLetterN, callData) < 0)
	Message(scn, "Error in writing letter, letter may not have been written", 
			MSG_CLEAR_DEFAULT);
  else
	Message(scn, FmtString1("Letter written to file %s", (String)callData), 
			MSG_CLEAR_DEFAULT);
}

void 
WriteLetterCallback(widget, clientData, callData)
	 Widget    widget;
	 XtPointer clientData;
	 XtPointer callData;
{
  SCREEN          *scn = (SCREEN*)clientData;
  if (DialogInputAndDispatch(scn, WriteLetterCallback1, clientData) < 0)
	return;
  Message(scn, "Enter full file name to write letter(s) to above", 
		  MSG_CLEAR_DEFAULT);
}

int 
LetterInfo(scn)
	 SCREEN *scn;
{
  LETTER          *letter;
  String          message;

  if (scn->folder.curLetterN < 0) 
	{Message(scn, "No letter selected", MSG_CLEAR_DEFAULT); return -1;}

  CheckIfLetterOk(scn, scn->folder.curLetN,-1);
  letter = scn->letterList[scn->folder.curLetN];
  message = FmtString("", "", "", "");

  sprintf(message, "Current letter is #%d of %d (folder %s)%s%s%s%s",
		  scn->folder.curLetterN + 1,
		  scn->folder.numLetters,
		  scn->folder.name,
		  letter->read ? ", read" : ", unread",
		  letter->replied ? ", replied" : "",
		  letter->deleted ? ", deleted" : "",
		  letter->tagged ? ", tagged" : "");
  Message(scn, message, MSG_CLEAR_DEFAULT);
  return 0;
}

void 
LetterInfoCallback(widget, clientData, callData)
	 Widget    widget;
	 XtPointer clientData;
	 XtPointer callData;
{
  LetterInfo((SCREEN*)clientData);
}
