/*---------------------------------------------------------------------------+
|	This file is part of the MuMail, Copyright (c) 1992-1993 by
|	Muhammad M. Saggaf. All rights reserved.
|
|	See the file COPYING (1-COPYING) or the manual page mumail(1)
|	for a full statement of rights and permissions.
+---------------------------------------------------------------------------*/

#include <X11/Intrinsic.h>
#include <X11/StringDefs.h>
#include <X11/Xaw/AsciiText.h>
#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <string.h>
#include <ctype.h>
#include <time.h>
#include "MuGeneric.h"
#include "MuGeneric.h"
#include "MmDecl.h"

time_t          ArpaToUnixDate();

char*
GetNextLetterStart(mailBuffer)
	 char *mailBuffer;
{
  static char *curBufPos;
  char        *bufPtr;

  if (mailBuffer == NULL) mailBuffer = curBufPos;
  if ((bufPtr = strstr(mailBuffer, "\n\nFrom ")))
	{curBufPos = bufPtr + 7; return bufPtr + 2;}
  return NULL;
}

char*
GetNextLetter(mailBuffer)
	 char *mailBuffer;
{
  static char *nextLetterStart = NULL;
  char        *letterStart;

  if (mailBuffer == NULL)
	letterStart = nextLetterStart;
  else if ((letterStart = GetNextLetterStart(mailBuffer)) == NULL) 
	return (nextLetterStart = NULL);

  if ((nextLetterStart = GetNextLetterStart(NULL)))
	*(nextLetterStart - 2) = CNULL;

  return letterStart;
}

LETTER*
ExtractLetterComponents(rawLetter)
	 char *rawLetter;
{
  static LETTER      *letter;
  String             headersBuf,
                     headersEnd,
                     token,
                     tmpPtr;
  int                n, i, tokenLen;
  Boolean            found;

  struct _header {
	String    pointer;
	Boolean   recognized;
  } header[MAX_HEADERS];

  if (rawLetter == NULL) return NULL;

  letter = XtNew(LETTER);
  letter->totalLength = strlen(rawLetter) + 1;

  /* Isolate the headers */
  if ((headersEnd = strstr(rawLetter, "\n\n"))) *headersEnd = CNULL;
  letter->content = headersEnd + 2;
  letter->deleted = False;
  headersBuf = XtNewString(rawLetter);

  /* Parse the headers buffer into strings */
  for (n = 0; (header[n].pointer = strtok((n ? NULL : headersBuf), "\n")); 
	   header[n].recognized = False, n++);

  letter->delimiter = header[0].pointer;
  header[0].recognized = True;

  tokenLen = strlen((token = FROM_TITLE));
  for (i = n - 1; i >= 0; i--)
	if (header[i].recognized == False &&
		!strncmp(header[i].pointer, token, tokenLen)) {
	  header[i].recognized = True;
	  letter->from = header[i].pointer + tokenLen + 1;
	  break;
	}
  if (i < 0) {letter->from = ""; found = False;}
  else found = True;

#define FindField(tokenString, var) { \
  tokenLen = strlen((token = tokenString)); \
  for (i = 0; i < n; i++) \
	if (header[i].recognized == False && \
		!strncmp(header[i].pointer, token, tokenLen)) { \
	  header[i].recognized = True; \
	  var = header[i].pointer + tokenLen + 1; \
	  break; \
	} \
  if (i >= n) {var = ""; found = False;} \
  else found = True;\
  }

  FindField(TO_TITLE, letter->to);
  FindField(SUBJECT_TITLE, letter->subject);
  FindField(DATE_TITLE, letter->date.fullS);

  FindField(READ_TITLE, letter->readS); letter->read = found;
  FindField(REPLIED_TITLE, letter->repliedS); letter->replied = found;

  letter->tagged = False;
  letter->retag = False;

  /* Begin change */

  tmpPtr = letter->delimiter = strcpy(rawLetter, letter->delimiter);
  tmpPtr = letter->from = strcpy(strend(tmpPtr) + 1, letter->from);
  tmpPtr = letter->to = strcpy(strend(tmpPtr) + 1, letter->to);
  tmpPtr = letter->subject = strcpy(strend(tmpPtr) + 1, letter->subject);
  tmpPtr = letter->date.fullS = strcpy(strend(tmpPtr) + 1, letter->date.fullS);
  letter->date.timeU = ArpaToUnixDate(tmpPtr);

  tmpPtr = letter->readS = strcpy(strend(tmpPtr) + 1, letter->readS);
  tmpPtr = letter->repliedS = strcpy(strend(tmpPtr) + 1, letter->repliedS);

  tmpPtr = letter->otherHeaders = strend(tmpPtr) + 1;  
  for (i = 0; i < n; i++)
	if (header[i].recognized == False) {
	  sprintf(tmpPtr, "%s\n", header[i].pointer);
	  tmpPtr = strend(tmpPtr);
	}
  *--tmpPtr = CNULL;

  XtFree(headersBuf);
  return letter;
}

int
GetLettersFromFolder(scn, folderName)
	 SCREEN *scn;
	 String folderName;
{
  FILE           *folder;
  String          rawLetter, tmpPtr;
  int             i;

  OpenStream(folder, FileInMailDir(folderName), "r", -1);

  /* free previous folder buffer */
  if (scn->folder.buffer) XtFree(scn->folder.buffer);
  tmpPtr = CopyStreamToBuffer(folder);

  /* Add 1 for NULL-termination, 2 for \n\n */
  scn->folder.buffer = (String)XtMalloc(strlen(tmpPtr) + 3);
  strcpy(scn->folder.buffer, "\n\n");
  strcat(scn->folder.buffer, tmpPtr);

  /* Free temporary buffer*/
  XtFree(tmpPtr);
  fclose(folder);

  FreeList((XtPointer)scn->letterList);
  for (i = 0; 
	   (rawLetter = GetNextLetter(i ? NULL : scn->folder.buffer)); 
	   i++)
	scn->letterList[i] = ExtractLetterComponents(rawLetter);

  scn->letterList[i] = NULL;
  scn->folder.numLetters = i;

  return 0;
}

void
MakeLetterIndex(scn)
	 SCREEN *scn;
{
  LETTER         *letter;
  char            dispItemsBuf[BUF256];
  String          tmpPtr, fromBuf, partCont, subjectBuf;
  char            date[BUF10];
  int             i, numLetters, numTagged, curLetN;

  /* Add 1 in case the folder is empty */
  if (scn->tocS) XtFree(scn->tocS);
  scn->tocS = (String)XtMalloc(scn->folder.numLetters * BUF256 + 1);

  tmpPtr = strcpy(scn->tocS, ""); 

  for (i = 0, numTagged = numLetters = 0, curLetN = -1;
	   (letter = scn->letterList[i]); i++) {

	if (letter->deleted) continue;

	strftime(date, 6, "%m/%d", localtime(&letter->date.timeU));

	fromBuf = *letter->from ? letter->from : *letter->to ? 
	  FmtString2("%s %s", TO_TITLE, letter->to) : "<<unknown>>";

	if (strlen(letter->subject) >= 52) subjectBuf = letter->subject;
	else {
	  partCont = strncpy(FmtString0(""), letter->content, 52);
	  partCont[52] = CNULL;
	  subjectBuf = FmtString2("%s <<%s>>", letter->subject, 
						   StripStringB(FilterString(partCont, "\n", SPACE)));
	}
	
	sprintf(dispItemsBuf, "%3d%1.1s %-5.5s %-17.17s  %-52.52s%1c", 
			numLetters + 1,

			letter->deleted ? TOC_DEL_FLAG : 
			letter->replied ? TOC_REPL_FLAG : 
			letter->read ? TOC_READ_FLAG : TOC_UNREAD_FLAG, 
			
			date,
			fromBuf, 
			subjectBuf,
			scn->letterList[i+1] ? NEWLINE : SPACE);

	strcat(tmpPtr, dispItemsBuf);
	tmpPtr = strend(tmpPtr);

	scn->letterList[numLetters] = letter;
	if (curLetN < 0 && !(letter->read)) curLetN = numLetters;
	scn->letterList[numLetters]->number = numLetters;
	if (letter->tagged) numTagged++;
	++numLetters;
  }

  scn->letterList[numLetters] = NULL;
  scn->folder.numLetters = numLetters;
  scn->folder.numTagged = numTagged;

  /* If there are any letters in the folder, we want the first unread 
	 one to be the current */
  scn->folder.curLetN = curLetN < 0 ? numLetters ? 0 : -1 : curLetN;
}

String
ConstructLetter(letter, letterMakeFormat)
	 LETTER *letter;
	 int     letterMakeFormat;
{
  String          letterBuffer,
                  curStartPos,
                  date;

  /* Added 256 just to be safe (for read, replied, ..etc.) */
  letterBuffer = (String)XtMalloc(letter->totalLength + 256);

  curStartPos = strcpy(letterBuffer, "");
  switch (letterMakeFormat) {

  case LETTER_MAKE_WITH_DELIMITER:

	strcat(curStartPos, FmtString1("%s\n", letter->delimiter));

  case LETTER_MAKE_WHOLE:
	date = CurrentLocalTimeString();

	if (letter->replied)
	  strcat(curStartPos, FmtString2("%s %s\n", REPLIED_TITLE, 
		*(letter->repliedS) ? letter->repliedS : date));
	if (letter->read)
	  strcat(curStartPos, FmtString2("%s %s\n", READ_TITLE, 
        *(letter->readS) ? letter->readS : date));

	strcat(curStartPos, FmtString1("%s\n", letter->otherHeaders));
	curStartPos = strend(curStartPos);
								
  case LETTER_MAKE_PARTIAL:
	sprintf(curStartPos, "%s %s\n%s %s\n%s %s\n%s %s\n",
			DATE_TITLE, letter->date.fullS,
			FROM_TITLE, letter->from,
			TO_TITLE, letter->to,
			SUBJECT_TITLE, letter->subject);
	curStartPos = strend(curStartPos);

  case LETTER_MAKE_BODY:
	sprintf(curStartPos, "\n%s", letter->content);
	curStartPos = strend(curStartPos);

	strcat(curStartPos, "\n\n");
  }

/*  *(curStartPos-1) = CNULL; leave it out, we want to keep the last newline*/
  return letterBuffer;
}

int
SaveFolder(scn)
  SCREEN *scn;
{
  LETTER          *letter;
  FILE            *folder;
  String          letterBuffer;
  int             i;

  OpenStream(folder, FileInMailDir(scn->folder), "w", -1);

  for (i = 0; (letter = scn->letterList[i]); i++) {
	/* Don't save if deleted */
	if (letter->deleted) continue;

	letterBuffer = ConstructLetter(letter, LETTER_MAKE_WITH_DELIMITER);
	fprintf(folder, "%s", letterBuffer);
	XtFree(letterBuffer);
  }

  fclose(folder);
  scn->folder.changed = False;
  return 0;
}

String
FindHeaderFieldFromFile(fileName, header)
	 String fileName,
	        header;
{
  FILE        *fp;
  static char fileS[BUF256];
  int         len = strlen(header);

  OpenStream(fp, fileName, "r", NULL);
  
  while (fgets(fileS, BUF256, fp))
	if (strncmp(fileS, header, len) == 0) {
	  fclose(fp);
	  *(strend(fileS) - 1) = CNULL;
	  return fileS+len+1;
	}

  fclose(fp); return NULL;
}

String
SuppressUnwantedAddressComp(header)
	 String          header;
{
  String          curPtr;
  int             dQuote, 
                  sQuote, 
                  paren;

  if (header == NULL) return header;

  for (dQuote = 0, sQuote = 0, paren = 0, curPtr = header; 
	   *curPtr; curPtr++) {

	switch (*curPtr) {
	case '\"': 
	  dQuote = dQuote ? 0 : 1;
	  *curPtr = ' ';
	  break;
	case '\'': 
	  sQuote = sQuote ? 0 : 1;
	  *curPtr = ' ';
	  break;
	case '(':
	  if (dQuote == 0 && sQuote == 0) 
		{paren++; *curPtr = ' ';}
	  break;
	case ')': 
	  if (dQuote == 0 && sQuote == 0) 
		{paren--; *curPtr = ' ';}
	  break;
	case '<':
	case '>':
	  *curPtr = ' ';
	}
	  
	if (dQuote || sQuote || paren || *curPtr == ',') *curPtr = ' ';
  }

  return header;
}

String
BuildAddress(header)
	 String          header;
{
  Boolean         notValidHeader;
  static String   newHeader;
  String          curPtr,
                  lastSpacePtr, 
                  tmpPtr;

  if (header == NULL) return header;
  else header = SuppressUnwantedAddressComp(header);

  if (newHeader) XtFree(newHeader);
  newHeader = XtNewString(header);

  lastSpacePtr = curPtr = newHeader; 
  notValidHeader = True; 

  do {
	if (isspace(*curPtr) || *(curPtr) == CNULL) {
	  if (notValidHeader)
		for (tmpPtr = lastSpacePtr; tmpPtr != curPtr; tmpPtr++) 
		  *tmpPtr = SPACE;

	  notValidHeader = True;
	  lastSpacePtr = curPtr;
	}
	  
	else if (*curPtr == '@' || *curPtr == '!')
	  notValidHeader = False;

  } while (*curPtr++);


  return newHeader;
}

time_t
ArpaToUnixDate(arpaDate)
	 String          arpaDate;
{
  static String    arpaMonthName[] = {"Jan", "Feb", "Mar", "Apr", "May", 
										"Jun", "Jul", "Aug", "Sep", "Oct",
										"Nov", "Dec", NULL};
  static char      monthName[BUF10];
  static struct tm tm;
  int              i;

  /* Like Thu, 15 Apr 93 08:09:47 -0500 */

  sscanf(arpaDate, " %*s %d %5s %d %d:%d:%d", &tm.tm_mday, monthName, 
		 &tm.tm_year, &tm.tm_hour, &tm.tm_min, &tm.tm_sec);

  if (tm.tm_year > 99) tm.tm_year = tm.tm_year - 1900;

  for (i = 0; arpaMonthName[i] &&
	   strcmp(arpaMonthName[i], monthName); i++);
  tm.tm_mon = i;

  tm.tm_isdst = -1;

  return mktime(&tm);
}

