/*---------------------------------------------------------------------------+
|	This file is part of MuMail, Copyright (c) 1992-1993 by
|	Muhammad M. Saggaf. All rights reserved.
|
|	See the file COPYING (1-COPYING) or the manual page mumail(1)
|	for a full statement of rights and permissions.
+---------------------------------------------------------------------------*/

#include <X11/Intrinsic.h>
#include <X11/StringDefs.h>
#include <X11/Shell.h>
#include <X11/Xaw/AsciiText.h>
#include "MmDecl.h"
#include "MuWin.h"
#include "MuGeneric.h"
#include "version.h"

#ifndef HELPFILE
#define HELPFILE "/usr/lib/X11/mumail.help"
#endif

extern void
                HighlightToEndOFLineAction(),
               
                CloseWindowAction(),
                HighlightCurrentLineAction(),

                OpenFolderAction(),
                SaveFolderAction(),
                QuickExitAction(),
                ExitAction(),

                TagUntagLetterAction(),
                RetagLettersAction(),
                DeleteLetterAction(),
                UndeleteLetterAction(),
                ComposeLetterAction(),
                ReplyToLetterAction(),
                ForwardLetterAction(),
                ExternalComposeLetterAction(),
                CopyLetterAction(),
                RefileLetterAction(),
                WriteLetterAction(),
                LetterInfoAction(),

                QuoteScStyleFmtAction(),
                QuoteScStyleUnfmtAction(),
                QuoteRnStyleAction(),
                InsertSignatureAction(),
                CancelDraftAction(),

                DisplayCurLetterPartialAction(),
                DisplayCurLetterWholeAction(),
                DisplayInsertionPointPosAction();

SCREEN          *AddScreen();
void
                DisplayLogo();


static char     version[] = "$Revision: 1.2a $";

SCREEN          *scnList[5];
int             curScnIndex,
                numScreens = 0;

void
main(argc, argv)
     int             argc;
     char           *argv[];
{
  XtAppContext    appContext;  
  Widget          topLevel;
  SCREEN          *firstScn;

  static char    *fallbackResources[] = {
#include "MuMail.ad.h"
    NULL,
  };

  static XrmOptionDescRec optionList[] =  {
    {"-modems", "modems", XrmoptionSepArg, NULL},
    {"-script", "script", XrmoptionSepArg, NULL},
    {"-entries", "defaultPhoneEntries", XrmoptionSepArg, NULL},
    {"-dial", "dialAutoStart", XrmoptionNoArg, "True"},
    {"-nodial", "dialAutoStart", XrmoptionNoArg, "False"},
    {"-emulator", "emulator", XrmoptionSepArg, ""},
    {"-noemulator", "noemulator", XrmoptionNoArg, ""},
    {"-nodefargs", "nodefargs", XrmoptionNoArg, ""},
  };

  static XtActionsRec actionTable[] = {
    {"ClearCutBuffer", ClearCutBufferAction},
    {"HighlightToEndOFLine", HighlightToEndOFLineAction},
    {"IncCurLetterNumber", IncCurLetterNumberAction},
    {"DecrCurLetterNumber", DecrCurLetterNumberAction},

    {"CloseWindow", CloseWindowAction},
    {"HighlightCurrentLine", HighlightCurrentLineAction},

    {"OpenFolder", OpenFolderAction},
    {"SaveFolder", SaveFolderAction},
    {"QuickExit", QuickExitAction},
    {"Exit", ExitAction},


    {"SetCurLetterNumber", SetCurLetterNumberAction},
    {"DisplayCurLetterPartial", DisplayCurLetterPartialAction},
    {"DisplayCurLetterWhole", DisplayCurLetterWholeAction},
    {"DisplayInsertionPointPos", DisplayInsertionPointPosAction},

    {"TagUntagLetter", TagUntagLetterAction},
    {"RetagLetters", RetagLettersAction},
    {"DeleteLetter", DeleteLetterAction},
    {"UndeleteLetter", UndeleteLetterAction},
    {"ComposeLetter", ComposeLetterAction},
    {"ReplyToLetter", ReplyToLetterAction},
    {"ForwardLetter", ForwardLetterAction},
    {"ExternalComposeLetter", ExternalComposeLetterAction},
    {"CopyLetter", CopyLetterAction},
    {"RefileLetter", RefileLetterAction},
    {"WriteLetter", WriteLetterAction},
    {"LetterInfo", LetterInfoAction},

    {"QuoteScStyleFmt", QuoteScStyleFmtAction},
    {"QuoteScStyleUnfmt", QuoteScStyleUnfmtAction},
    {"QuoteRnStyle", QuoteRnStyleAction},
    {"InsertSignature", InsertSignatureAction},
    {"CancelDraft", CancelDraftAction},
  };

  topLevel = XtAppInitialize(&appContext, "MuMail", optionList,
							 XtNumber(optionList), &argc, argv,
							 fallbackResources, NULL, 0);
  if (argc > 1)
	{MuError(FmtString1("Wrong argument: %s", argv[1])); exit(1);}
  MuNotice("MuMail is initializing..");  

  XtAppAddActions(appContext, actionTable, XtNumber(actionTable));
  XtOverrideTranslations(topLevel,
		       XtParseTranslationTable("<Message>WM_PROTOCOLS: QuickExit()"));

  firstScn = AddScreen(topLevel, INFOLDER_NAME);
  XtAppMainLoop(appContext);
}

Widget
CreateTocW(parentW)
	 Widget parentW;
{
  Widget widget;
  static XawTextSelectType selectTypes[] = {XawselectLine, XawselectNull};

  widget = AddAsciiText("tocW", parentW);
  XtVaSetValues(widget, XtNselectTypes, selectTypes, NULL);
  return widget;
}

Widget
CreateBodyW(parentW)
	 Widget parentW;
{
  Widget          widget;
  Widget          textSinkW;
  XFontStruct    *font;
  Dimension       width;

  widget = AddAsciiText("bodyW", parentW);

  XtVaGetValues(widget, XtNfont, &font, XtNtextSink, 
				&textSinkW,	NULL);
  width = font->max_bounds.rbearing + font->min_bounds.lbearing;
  XtVaSetValues(widget, XtNwidth, 81 * width, NULL);
  XtVaSetValues(widget, XtNrightMargin, 10 * width, NULL); 
  return widget;

/*  XtVaSetValues(widget, XtNwidth, 81 * width, XtNleftMargin, 
				width, XtNrightMargin, 10 * width, NULL); 

  XtVaSetValues(textSinkW, XtNwidth, (Dimension)486, 
				XtNleftMargin, 
				(Dimension) 6 , XtNrightMargin,  (Dimension) 60, NULL); 
  printf("-%d-\n", (int)width);*/
}

void
CreateDialogW(scn, parentW)
	 SCREEN          *scn;
	 Widget          parentW;
{
  Widget boxW = AddPaned("dialogW", parentW);

  scn->dialog.yesW = AddButton("yesW", boxW, NULL, NULL);
  scn->dialog.cancelW = AddButton("cancelW", boxW, NULL, NULL);
  scn->dialog.valueW = AddAsciiText("valueW", boxW);

  XtVaSetValues(scn->dialog.yesW, XtNmappedWhenManaged, False, NULL);
  XtVaSetValues(scn->dialog.cancelW, XtNmappedWhenManaged, False, NULL);

  scn->dialog.active = False;
}

void
SetIcon(parentW)
     Widget          parentW;
{
#include "MuMail.icon.h"

  Pixmap icon;

  icon = 
	XCreateBitmapFromData(XtDisplay(parentW),
						  DefaultRootWindow(XtDisplay(parentW)),
						  icon_bits, icon_width, icon_height);
  XtVaSetValues(GetShell(parentW), XtNiconPixmap, icon, NULL);
}

void
DisplayLogo(scn)
	 SCREEN   *scn;
{
  static char logo[] =
#include "MuMail.logo.h"
	;
  char        versionInfo[BUF256];
  String      logoBuffer; 

  sprintf(versionInfo, "%5sVersion %s rev. %s %s-%s %s@%s %s %s\n\n\n%12s%s", 
		  "", VERSION, REVISION, COMPILE_HOSTTYPE, COMPILE_OSNAME, 
		  COMPILE_BY, COMPILE_HOST, COMPILE_DATE, COMPILE_TIME, "",
		  "Comments, suggestions, and bug fixes are welcome");
  
  logoBuffer = (String)XtMalloc(strlen(logo)+strlen(versionInfo));
  sprintf(logoBuffer, "%s%s", logo, versionInfo);

  DisplayBufferInBodyBox(scn, logoBuffer);
  XtFree(logoBuffer);
}

void
DisplayLogoCallback(widget, clientData, callData)
	 Widget    widget;
	 XtPointer clientData;
	 XtPointer callData;
{
  DisplayLogo((SCREEN*)clientData);
}

void
DisplayHelpFileCallback(widget, clientData, callData)
	 Widget    widget;
	 XtPointer clientData;
	 XtPointer callData;
{
  DisplayFileInBodyBox((SCREEN*)clientData, HELPFILE);
}  

String
XMailerMessage()
{
  /* Instead of allocating new space */
  String buffer = FmtString0(""); 

  sprintf(buffer, "X-Mailer: MuMail [version %s.%s %s-%s]\n", 
		  VERSION, REVISION, COMPILE_HOSTTYPE, COMPILE_OSNAME);
  return buffer;
}


