/*---------------------------------------------------------------------------+
|	This file is part of MuMail, Copyright (c) 1992-1993 by
|	Muhammad M. Saggaf. All rights reserved.
|
|	See the file COPYING (1-COPYING) or the manual page mumail(1)
|	for a full statement of rights and permissions.
+---------------------------------------------------------------------------*/

#include <X11/Intrinsic.h>
#include <X11/StringDefs.h>
#include <X11/Shell.h>
#include "MmDecl.h"
#include "MuWin.h"
#include "MuGeneric.h"

extern Widget
                CreateTocW(),
                CreateBodyW();

extern void
                SetIcon(),
                CreateDialogW();

SCREEN*
AddScreen(topLevelW, folderName)
	 Widget          topLevelW;
	 String          folderName;
{
  SCREEN          *scn;
  Widget          topPaned;
  static Atom     WM_DELETE_WINDOW;
  String          title;
  int             i;

  /* Find the lowest available screen slot */
  for (i = 0; i < numScreens; i++)
	if (scnList[i] == NULL) break;

  scn = scnList[i] = XtNew(SCREEN);
  scn->number = i;
  numScreens++;

  /*-----*/

  title = FmtString1("Mumail: Screen %d", scn->number+1);
  scn->parentW =
	XtVaCreatePopupShell(FmtString1("screen%d", scn->number+1), 
						 topLevelShellWidgetClass, topLevelW, XtNtitle, title,
						 XtNiconName, title, NULL);
  XtOverrideTranslations(scn->parentW,
		     XtParseTranslationTable("<Message>WM_PROTOCOLS: CloseWindow()"));
  SetIcon(scn->parentW);

  /*-----*/

  scn->topLevelW = topLevelW;

  topPaned = AddPaned("mainPaned", scn->parentW);
  CreateMainMenu(scn, topPaned);
  scn->tocW = CreateTocW(topPaned);
  CreateDialogW(scn, topPaned);
  scn->bodyW = CreateBodyW(topPaned);
  scn->labelW = AddLabel("labelW", topPaned);

  scn->folder.name = XtNewString(folderName);
  scn->folder.buffer = NULL;

  /* We have to set this here so that they are defined even if we fail 
	 to open the initial folder */

  scn->folder.numLetters = 0;
  scn->folder.curLetterN = -1;
  scn->folder.changed = False;

  scn->letterList[0] = NULL;
  scn->draft.name = XtNewString(FmtString2("%s.%d", DRAFT_NAME, 
											  scn->number+1));
  scn->draft.active = False;
  scn->tocS = NULL;

  /*-----*/

  DisplayLogo(scn);
  Message(scn, FmtString("Welcome to MuMail version %s.%s", VERSION, 
							REVISION, ""), MSG_CLEAR_NEVER);
  /*-----*/
  
  XtPopup(scn->parentW, XtGrabNone);
  WM_DELETE_WINDOW = XInternAtom(XtDisplay(scn->parentW), 
								 "WM_DELETE_WINDOW", False);
  XSetWMProtocols(XtDisplay(scn->parentW), XtWindow(scn->parentW), 
				  &WM_DELETE_WINDOW,  1);

  /*-----*/

  OpenFolder(scn, scn->folder.name);
  return scn;
}

SCREEN*
NextScreen(scn)
	 SCREEN          *scn;
{
  int          i;

  for (i = scn->number+1; i < MAX_SCREENS; i++)
	if (scnList[i]) return scnList[i];

  return (SCREEN*)NULL;
}

SCREEN*
WidgetToScn(widget)
	 Widget widget;
{
  int          i;

  widget = GetShell(widget);
  for (i = 0; i < MAX_SCREENS; i++)
	if (scnList[i] && scnList[i]->parentW == widget) return scnList[i];

  MuError("(SERIOUS ERROR): Could not find screen from widget");
  return NULL;
}

SCREEN*
NewWindowProc(scn)
	 SCREEN          *scn;
{
  return AddScreen(scn->topLevelW, scn->folder.name);
}

void
NewWindowCallback(widget, clientData, callData)
	 Widget    widget;
	 XtPointer clientData;
	 XtPointer callData;
{
  NewWindowProc((SCREEN*)clientData);
}

/*---------------------------------------------------------------------------+
| Close window routines.
+---------------------------------------------------------------------------*/

void
CloseWindow(scn)
	 SCREEN          *scn;
{
  if (numScreens == 1) {QuickExit(scn); return;}

  XtUnmapWidget(scn->parentW);
  XtDestroyWidget(scn->parentW);
  
  numScreens--;
  scnList[scn->number] = NULL;
  XtFree((String)scn);
}

void 
CloseWindowCallback1_1(widget, clientData, callData)
	 Widget    widget;
	 XtPointer clientData;
	 XtPointer callData;
{
  SCREEN          *scn = (SCREEN*)clientData;
  CloseWindow(scn);
}

void 
CloseWindowCallback1(widget, clientData, callData)
	 Widget    widget;
	 XtPointer clientData;
	 XtPointer callData;
{
  SCREEN          *scn = (SCREEN*)clientData;
  
  if (numScreens == 1)
	if (BooleanInputAndDispatch(scn, CloseWindowCallback1_1, clientData) < 0)
	  return;
	else
	  Message(scn, "Last window. Exit program?", MSG_CLEAR_DEFAULT);
  else
	CloseWindowCallback1_1(widget, clientData, callData);
}

void 
CloseWindowCallback(widget, clientData, callData)
	 Widget    widget;
	 XtPointer clientData;
	 XtPointer callData;
{
  SCREEN          *scn = (SCREEN*)clientData;

  if (scn->folder.changed)
	if (BooleanInputAndDispatch(scn, CloseWindowCallback1, clientData) < 0)
	  return;
	else
	  Message(scn, "Changes were made to current folder. Discard them?", 
			  MSG_CLEAR_DEFAULT);
  else
	CloseWindowCallback1(widget, clientData, callData);
}
