/*---------------------------------------------------------------------------+
|	This file is part of MuMail, Copyright (c) 1992-1993 by
|	Muhammad M. Saggaf. All rights reserved.
|
|	See the file COPYING (1-COPYING) or the manual page mumail(1)
|	for a full statement of rights and permissions.
+---------------------------------------------------------------------------*/

#include <X11/Intrinsic.h>
#include <X11/StringDefs.h>
#include <X11/Xaw/AsciiText.h>
#include <string.h>
#include "MuWin.h"
#include "MuGeneric.h"
#include "MmDecl.h"

#define MakeTocEditable(scn)\
  XtVaSetValues(scn->tocW, XtNeditType, XawtextEdit, NULL);
#define MakeTocReadOnly(scn)\
  XtVaSetValues(scn->tocW, XtNeditType, XawtextRead, NULL);

extern XawTextPosition
                FindStartOfCurrentLine(),
                TextGetNthLineStartPos();

int
PutTextToTocDisplay(scn, start, text)
	 SCREEN           *scn;
	 XawTextPosition   start;
	 String            text;
{
  XawTextPosition          startPos, endPos;
  XawTextBlock             textBlock;
  String                   tocS;
  int                      retStatus;

  XawTextGetSelectionPos(scn->tocW, &startPos, &endPos);

  SetupTextBlock(textBlock, text);
  retStatus = XawTextReplace(scn->tocW, start, start+textBlock.length, 
							 &textBlock);

  XawTextSetSelection(scn->tocW, startPos, endPos);

  XtVaGetValues(scn->tocW, XtNstring, &tocS, NULL);
  XtFree(scn->tocS);
  scn->tocS = XtNewString(tocS);

  return (retStatus == XawEditDone ? 0 : -1);
}

void
TocMoveToLetterLine(scn, letterN)
	 SCREEN          *scn;
	 int             letterN;
{
  XawTextSetInsertionPoint(scn->tocW, TextGetNthLineStartPos(scn->tocW, 
															 letterN+1));
}

void
TocHighlightLetter(scn, letterN)
	 SCREEN          *scn;
	 int             letterN;
{
  TocMoveToLetterLine(scn, letterN);
  HighlightCurrentLine(scn->tocW);
}

int
TocSetLetterFlag(scn, letterN, flag)
	 SCREEN          *scn;
	 int             letterN; 
	 String          flag;
{
  TocHighlightLetter(scn, letterN);
  return PutTextToTocDisplay(scn, FindStartOfCurrentLine(scn->tocW) + 
							 TOC_FLAG_POS, flag);
}

void
RedisplayToc(scn)
	 SCREEN *scn;
{
  XawTextDisableRedisplay(scn->tocW);
  XtVaSetValues(scn->tocW, XtNstring, scn->tocS, NULL);
  XawTextSetSource(scn->tocW, XawTextGetSource(scn->tocW), 
 		          TextGetNthLineStartPos(scn->tocW, scn->folder.curLetN+1));
  TocHighlightLetter(scn, scn->folder.curLetterN);
  XawTextEnableRedisplay(scn->tocW);
}

int
GetCurLetterNumberFromToc(scn)
	 SCREEN *scn;
{
  char    *tocS;
  int     letterN;

  tocS = GetCurrentTextSelection(scn->tocW, BUF10);
  if (sscanf(tocS, " %d", &letterN) == 1) return letterN;
  else return -1;
}

int
SetCurLetterNumber(scn)
	 SCREEN          *scn;
{
  int     letterN;

  letterN = GetCurLetterNumberFromToc(scn);
  scn->folder.curLetterN = letterN < 0 ? -1 : letterN - 1;

  if (1) LetterInfo(scn);

/*  printf("-%d-\n", scn->folder.curLetN);*/
  return letterN;
}

void
ScrollTocToLine(scn, lineN)
	 SCREEN          *scn;
	 int             lineN;
{
  ScrollTextToLine(scn->tocW, lineN);
}




