/*---------------------------------------------------------------------------+
|	This file is part of MuMail, Copyright (c) 1992-1993 by
|	Muhammad M. Saggaf. All rights reserved.
|
|	See the file COPYING (1-COPYING) or the manual page mumail(1)
|	for a full statement of rights and permissions.
+---------------------------------------------------------------------------*/

#include "config.h"
#include <X11/Intrinsic.h>
#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <string.h>
#include <errno.h>
#include <ctype.h>
#include <time.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <sys/wait.h>
#include "MuGeneric.h"
#include "MmDecl.h"

/*---------------------------------------------------------------------------+
| Stream/buffer routines.
+---------------------------------------------------------------------------*/
  
String
CopyStreamToBuffer(stream)
	 FILE *stream;
{
  int            fd = fileno(stream);
  Cardinal       fileSize;
  static char   *buffer;
  struct stat    statInfo;
  
  if (fstat(fd, &statInfo) < 0) {
	MuPError("Could not stat file");
	return NULL;
  }

  /* Add 1 for NULL-termination */
  buffer = XtMalloc((fileSize = (Cardinal)(statInfo.st_size)) + 1);

  if (read(fd, buffer, (int)fileSize) < 0) {
	MuPError("Could not read from file");
	XtFree(buffer);
	return NULL;
  }
  
  buffer[fileSize] = CNULL;
  return buffer;
}

String
CopyFileToBuffer(fileName)
	 String          fileName;
{
  FILE            *fp;
  String          buffer;

  OpenStream(fp, fileName, "r", NULL);
  buffer = CopyStreamToBuffer(fp);
  fclose(fp);
  return buffer;
}

int
AppendBufferToStream(stream, buffer)
	 FILE   *stream;
	 String buffer;
{
  int            fd = fileno(stream);

  if (fseek(stream, 0, SEEK_END) < 0) return -1;
  if (write(fd, buffer, strlen(buffer)) < 0)
	{MuPError("Could not append buffer to stream");
	 return -1;}
  return 0;
}

int
AppendBufferToFile(fileName, buffer)
	 String fileName;
	 String buffer;
{
  FILE          *fp;
  int           retStatus;

  OpenStream(fp, fileName, "a", -1);
  retStatus = AppendBufferToStream(fp, buffer);
  fclose(fp);
  return retStatus;
}

int
AppendFileToStream(file, stream)
	 String          file;
	 FILE            *stream;
{
  String        buffer;
  int           retStatus;

  if ((buffer = CopyFileToBuffer(file)) == NULL) return -1;
  retStatus = AppendBufferToStream(stream, buffer);
  XtFree(buffer);
  return retStatus;
}

int
AppendFileToFile(file1, file2)
	 String file1;
	 String file2;
{
  String        buffer;
  int           retStatus;

  if ((buffer = CopyFileToBuffer(file1)) == NULL) return -1;
  retStatus =  AppendBufferToFile(file2, buffer);
  XtFree(buffer);
  return retStatus;
}

String
PrependToBuffer(buffer, string)
	 String          buffer,
                     string;
{
  String          newBuf;

  newBuf = (String)XtMalloc(strlen(buffer)+strlen(string)+1);
  strcat(strcpy(newBuf, string), buffer);

  XtFree(buffer);
  return newBuf;
}

/*---------------------------------------------------------------------------+
| Pipe routines.
+---------------------------------------------------------------------------*/

int
WriteToPipe(pd, data, size)
     int             *pd;
     String          data;
     int             size;
{
  int          retStatus;
  if ((retStatus = write(pd[1], data, size)) < 0)
    MuPError("Could not write to pipe");
  return retStatus;
}

int
ReadFromPipe(pd, data, size)
     int             *pd;
     String          data;
     int             size;
{
  int          retStatus;
  if ((retStatus = read(pd[0], data, size)) < 0)
    MuPError("Could not read from to pipe");
  return retStatus;
}

/*---------------------------------------------------------------------------+
| File routines.
+---------------------------------------------------------------------------*/

String
FileInHomeDir(fileName)
	 String fileName;
{
  static String homeDir = NULL;

  if (homeDir == NULL) homeDir = getenv("HOME");
  return FmtString2("%s/%s", homeDir, fileName);
}

String
FileInMailDir(fileName)
	 String fileName;
{
  return FileInHomeDir(FmtString1("Mail/%s", fileName));
}

int MuSystem(cmd)
	 String cmd;
{
  int retStatus = system(cmd);
  if (WEXITSTATUS(retStatus) == 0) return 0;
  else return -1;
}

int
CopyFile(source, dest)
	 String source;
	 String dest;
{
  /* This is pathetic */
  return MuSystem(FmtString2("cp %s %s", source, dest));
}

int
BackupFile(fileName)
	 String fileName;
{
  return CopyFile(fileName, FmtString1("%s~", fileName));
}

int
ExternalEditFile(fileName)
	 String  fileName;
{
  return MuSystem(FmtString2("%s %s", EXT_EDITOR, fileName));
}

/*---------------------------------------------------------------------------+
| Misc. routines.
+---------------------------------------------------------------------------*/

void
FreeList(listArr)
     XtPointer       listArr[];
{
  int             i;

  for (i = 0; listArr[i]; i++)
    XtFree(listArr[i]);

  listArr[0] = NULL;
}

String
CurrentLocalTimeString()
{
  time_t curTime = time((time_t*)0);
  String timeS = ctime(&curTime);
  *(strend(timeS) - 1) = CNULL;
  return timeS;
}

/*---------------------------------------------------------------------------+
| String routines.
+---------------------------------------------------------------------------*/

char*
FmtString(fmt, a, b, c)
	 char *fmt, *a, *b, *c;
{

  /* Why so? So that calls to this function can nested 20 levels deep */
  static char buffer[20][1024];
  static int  curBuffer = 19;

  if (--curBuffer < 0) curBuffer = 19;
  sprintf(buffer[curBuffer], fmt, a, b, c);
  return buffer[curBuffer];
}

String
strend(string)
	 char *string;
{
  return string + strlen(string);
}

/*---------------------------------------------------------------------------+
| FilterString - replaces unwanted characters with something else.
+---------------------------------------------------------------------------*/

String
FilterString(string, unwanted, replacement)
     String         string,
                    unwanted;
     char           replacement;
{
  int i, j;

  for (i = 0; string[i]; i++)
    for (j = 0; unwanted[j]; j++)
      if (string[i] == unwanted[j]) string[i] = replacement;

  return string;
}

/*---------------------------------------------------------------------------+
| StripStringB - removes leading white space from a string.
+---------------------------------------------------------------------------*/

String
StripStringB(string)
	 String          string;
{
  while (isspace(*string)) string++;
  return string;
}

/*---------------------------------------------------------------------------+
| strerror - for systems that do not have it.
+---------------------------------------------------------------------------*/

#if !HAVE_STRERROR
char*
strerror(err)
     int err;
{
  extern char *sys_errlist[];

  return sys_errlist[err];
}
#endif

int
GetNthLinePos(buffer, n)
	 String          buffer;
	 int             n;
{
  int          line, i;

  for (i = 0, line = 1; line < n; i++, line++)
	while (buffer[i] != '\n') i++;

  return i;
}

/*---------------------------------------------------------------------------+
| Error messages and notices.
+---------------------------------------------------------------------------*/

void
MuError(msg)
     char           *msg;
{
  fprintf(stderr, ">> Error: %s.\n", msg);
}

#if 0
void
se_warning(msg)
     char           *msg;
{
  char            buf[REG_BUF];

  sprintf(buf, "\r>> Warning: %s.", msg);
  show(buf);
}

void
se_warningf(fmt, a, b, c)
     char           *fmt,
                    *a,
                    *b,
                    *c;
{
  char            buf[REG_BUF];

  sprintf(buf, fmt, a, b, c);
  se_warning(buf);
}
#endif

void
MuNotice(msg)
     char           *msg;
{
  fprintf(stderr, ">> Notice: %s.\n", msg);
}

void
MuPError(msg)
     char           *msg;
{
  fprintf(stderr, ">> Error: %s: %s.\n", msg, strerror(errno));
}
